@@ -1,7 +1,7 @@
 /*
- * Programm XBLAST V1.2.10 or higher
+ * Programm XBLAST V2.1.10 or higher
  * (C) by Oliver Vogel (e-mail: vogel@ikp.uni-koeln.de)
- * March 6th 1996
+ * November 8th 1996
  * started August 1993
  *
  * File: graphics
@@ -32,6 +32,7 @@
 #include <X11/Intrinsic.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
 
 #define _GRAPHICS_C
 
@@ -44,6 +45,9 @@
 #include "score.h"
 #include "graphics.h"
 #include "setup.h"
+#include "event.h"
+#include "map.h"
+#include "main.h"
 
 #define BELL_VOLUME 80
 
@@ -57,63 +61,68 @@
 /*
  * global variables
  */
-int colorMode[MAX_PLAYER];
-Display *dpy[MAX_PLAYER];
+int colorMode[MAX_DISPLAY];
+Display *dpy[MAX_DISPLAY] = {NULL, NULL, NULL, NULL, NULL, NULL};
 
 /*
  * local Variables 
  */
-static Window win[MAX_PLAYER];
-static int def_depth[MAX_PLAYER];
-static int white_pixel[MAX_PLAYER];
-static int black_pixel[MAX_PLAYER];
+static Window win[MAX_DISPLAY];
+static int def_depth[MAX_DISPLAY];
+static int white_pixel[MAX_DISPLAY];
+static int black_pixel[MAX_DISPLAY];
 
 static char *XBlastResName  = "xblast";
 static char *XBlastResClass = "XBlast";
 
-static Pixmap pix[MAX_PLAYER];
-static Pixmap pix_block[MAX_PLAYER][MAX_BLOCK];
-static Pixmap pix_sprite_mask[MAX_PLAYER][MAX_ANIME];
-static Pixmap pix_sprite_bits[MAX_PLAYER][MAX_PLAYER][MAX_ANIME];
-static Pixmap pix_bomb_mask[MAX_PLAYER][MAX_BOMBS];
-static Pixmap pix_bomb_bits[MAX_PLAYER][MAX_BOMBS];
-static Pixmap pix_expl_mask[MAX_PLAYER][MAX_EXPLOSION];
-static Pixmap pix_expl_bits[MAX_PLAYER][MAX_EXPLOSION];
-static Pixmap pix_expl_block[MAX_PLAYER][MAX_EXPLOSION];
-static Pixmap pix_leds[MAX_PLAYER][2];
-static Pixmap pix_score[MAX_PLAYER][MAX_SCORE_TILES + MAX(MAX_LIVES,9)];
+static Pixmap pix[MAX_DISPLAY];
+static Pixmap pix_block[MAX_DISPLAY][MAX_BLOCK];
+static Pixmap pix_sprite_mask[MAX_DISPLAY][MAX_ANIME];
+static Pixmap pix_sprite_bits[MAX_DISPLAY][MAX_PLAYER][MAX_ANIME];
+static Pixmap pix_bomb_mask[MAX_DISPLAY][MAX_BOMBS];
+static Pixmap pix_bomb_bits[MAX_DISPLAY][MAX_BOMBS];
+static Pixmap pix_expl_mask[MAX_DISPLAY][MAX_EXPLOSION];
+static Pixmap pix_expl_bits[MAX_DISPLAY][MAX_EXPLOSION];
+static Pixmap pix_expl_block[MAX_DISPLAY][MAX_EXPLOSION];
+static Pixmap pix_leds[MAX_DISPLAY][2];
+static Pixmap pix_score[MAX_DISPLAY][MAX_SCORE_TILES];
 
 
-static GC gc_frompix[MAX_PLAYER];
-static GC gc_window[MAX_PLAYER];
-static GC gc_clearpix[MAX_PLAYER];
-static GC gc_drawblock[MAX_PLAYER];
-static GC gc_addon[MAX_PLAYER];
-static GC gc_sprite_mask[MAX_PLAYER];
-static GC gc_sprite_bits[MAX_PLAYER];
-static GC gc_text_black[MAX_PLAYER];
-static GC gc_text_white[MAX_PLAYER];
+static GC gc_frompix[MAX_DISPLAY];
+static GC gc_window[MAX_DISPLAY];
+static GC gc_clearpix[MAX_DISPLAY];
+static GC gc_drawblock[MAX_DISPLAY];
+static GC gc_addon[MAX_DISPLAY];
+static GC gc_sprite_mask[MAX_DISPLAY];
+static GC gc_sprite_bits[MAX_DISPLAY];
+static GC gc_text_black[MAX_DISPLAY];
+static GC gc_text_white[MAX_DISPLAY];
 
-static char *font_name[MAX_PLAYER][NUM_FONTS];
-static XFontStruct *font_struct[MAX_PLAYER][NUM_FONTS];
+static char *font_name[MAX_DISPLAY][NUM_FONTS];
+static XFontStruct *font_struct[MAX_DISPLAY][NUM_FONTS];
 
-static int winX[MAX_PLAYER],winY[MAX_PLAYER];
-static int winW[MAX_PLAYER],winH[MAX_PLAYER];
+static int winX[MAX_DISPLAY],winY[MAX_DISPLAY];
+static int winW[MAX_DISPLAY],winH[MAX_DISPLAY];
 
-static Sprite sprite_list[255];
-static Sprite *sprite_max = sprite_list;
-static XRectangle xrec[MAZE_W*MAZE_H];
+static XRectangle xrec[MAZE_W*MAZE_H + STAT_W*STAT_H];
 static XRectangle *xrec_max = xrec;
-static XBColorTriple col_triple[MAX_PLAYER][MAX_BLOCK];
 
-static int iconified[MAX_PLAYER] = { FALSE, FALSE, FALSE, FALSE };
+static XBColorTriple col_triple[MAX_DISPLAY][MAX_BLOCK];
+
+static int iconified[MAX_DISPLAY] = { FALSE, FALSE, FALSE, FALSE };
 
 
 /* Default Player Colors */
 
 static PlayerColor player_color [MAX_PLAYER];
 /* local variable for color cells */
-static PlayerColorCell player_color_cell[MAX_PLAYER][MAX_PLAYER] = {
+static PlayerColorCell player_color_cell[MAX_DISPLAY][MAX_PLAYER] = {
+  {
+    { 0,0,0,0,0,0},
+    { 0,0,0,0,0,0},
+    { 0,0,0,0,0,0},
+    { 0,0,0,0,0,0},
+  },
   {
     { 0,0,0,0,0,0},
     { 0,0,0,0,0,0},
@@ -144,7 +153,7 @@ static PlayerColorCell player_color_cell[MAX_PLAYER][MAX_PLAYER] = {
  * color string constants
  */
 
-static DisplayColor display_color[MAX_PLAYER];
+static DisplayColor display_color[MAX_DISPLAY];
 
 /*
  * local protoptypes
@@ -158,7 +167,8 @@ static Pixmap create_black_white_map(int disp, BitmapStruct *bitmap);
 static Pixmap create_pixmap(int disp, BitmapStruct *bitmap, int fg, int bg);
 static void init_fonts_color (int disp);
 static void init_fonts_bw (int disp);
-static void init_window(int disp);
+static void init_window_normal(int disp);
+static void init_window_override(int disp);
 static void init_player_colors(int disp);
 static void init_score_tiles_color(int disp);
 static void init_pixmap_color(int disp);
@@ -166,7 +176,7 @@ static void init_sprites_color(int disp);
 static void init_score_tiles_bw(int disp);
 static void init_pixmap_bw(int disp);
 static void init_sprites_bw(int disp);
-static void flush_sync(int disp, int num_disp);
+static void flush_sync(int disp);
 #else
 static void x_fatal();
 static void x_warning ();
@@ -175,7 +185,8 @@ static Pixmap create_black_white_map();
 static Pixmap create_pixmap();
 static void init_fonts_color ();
 static void init_fonts_bw ();
-static void init_window();
+static void init_window_normal();
+static void init_window_override();
 static void init_player_colors();
 static void init_score_tiles_color();
 static void init_sprites_color();
@@ -203,7 +214,7 @@ x_fatal (disp, text1, text2)
 #endif
 {
   fprintf(stderr, "ERROR: Display(%d), %s %s.\n", disp, text1, text2);
-  exit(1);
+  exit_prg(1);
 }
 
 
@@ -228,31 +239,6 @@ x_warning (disp, text1, text2)
 }
 
 
-#if 0
-/* 
- * local function: alloc_color 
- */
-#ifdef __STDC__
-static int 
-alloc_color(int disp, 
-	    char *color_name)
-#else
-static int 
-alloc_color (disp, color_name)
-     int disp;
-     char *color_name;
-#endif
-{
-  XColor colorUsed, colorExact;
-  Colormap colormap = DefaultColormap(dpy[disp], DefaultScreen(dpy[disp]));
-
-  if (!XAllocNamedColor(dpy[disp],colormap,color_name,&colorUsed, 
-			&colorExact)) {
-    x_fatal(disp, "alloc color cell (ro) failed for ",color_name);
-  }
-  return colorUsed.pixel;
-}
-#endif
 
 
 /* 
@@ -323,9 +309,18 @@ create_bitmap (disp, bitmap)
      BitmapStruct *bitmap;
 #endif
 {
+#ifdef DEBUG_X
+  Pixmap result;
+
+  result =XCreatePixmapFromBitmapData 
+    (dpy[disp], win[disp], (char *)bitmap->data, bitmap->width, bitmap->height,
+     1, 0, 1);
+  return result;
+#else
   return XCreatePixmapFromBitmapData 
     (dpy[disp], win[disp], (char *)bitmap->data, bitmap->width, bitmap->height,
      1, 0, 1);
+#endif
 }
 
 
@@ -336,7 +331,7 @@ create_bitmap (disp, bitmap)
 #ifdef __STDC__
 static Pixmap 
 create_black_white_map (int disp, 
-	       BitmapStruct *bitmap)
+			BitmapStruct *bitmap)
 #else
 static Pixmap 
 create_black_white_map (disp, bitmap)
@@ -344,6 +339,23 @@ create_black_white_map (disp, bitmap)
      BitmapStruct *bitmap;
 #endif
 {
+#ifdef DEBUG_X
+  Pixmap result;
+
+  if (1 != def_depth[disp])  {
+    result = XCreatePixmapFromBitmapData
+      (dpy[disp], win[disp], (char *)bitmap->data, 
+       bitmap->width, bitmap->height,
+       1, 0, 1);
+    
+  } else {
+    result = XCreatePixmapFromBitmapData 
+      (dpy[disp], win[disp], (char *)bitmap->data, 
+       bitmap->width, bitmap->height,
+       black_pixel[disp], white_pixel[disp], 1);
+  }
+  return result;
+#else
   if (1 != def_depth[disp])  {
     return XCreatePixmapFromBitmapData
       (dpy[disp], win[disp], (char *)bitmap->data, 
@@ -355,6 +367,7 @@ create_black_white_map (disp, bitmap)
        bitmap->width, bitmap->height,
        black_pixel[disp], white_pixel[disp], 1);
   }
+#endif
 }
 
 
@@ -535,7 +548,6 @@ init_score_tiles_color (disp)
 #endif
 {
   int i,j;
-  static char LivesText[10];
   int fg, bg;
   Pixmap tmp, pix_R, pix_G;
   PlayerColorCell *col = player_color_cell[disp];
@@ -555,6 +567,7 @@ init_score_tiles_color (disp)
     XFreePixmap(dpy[disp],tmp);
   }
   
+  /* init other score tiles */
   for (i=0; i < SBDead ; i++) {
     pix_score[disp][i] = create_pixmap(disp, &(score_tile[i]), fg,bg);
   }
@@ -572,41 +585,24 @@ init_score_tiles_color (disp)
       XSetStipple(dpy[disp], gc_addon[disp], pix_R);
       XSetForeground(dpy[disp], gc_addon[disp], col[j].helmet);
       XFillRectangle(dpy[disp], pix_score[disp][SBDead + i*MAX_PLAYER +j],
-		     gc_addon[disp], 0, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
+		     gc_addon[disp], 0, 0, STAT_WIDTH, STAT_HEIGHT);
       /* hand and feet */
       XSetStipple(dpy[disp], gc_addon[disp], pix_G);
       XSetForeground(dpy[disp], gc_addon[disp], col[j].hands_feet);
       XFillRectangle(dpy[disp], pix_score[disp][SBDead + i*MAX_PLAYER +j],
-		     gc_addon[disp], 0, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
+		     gc_addon[disp], 0, 0, STAT_WIDTH, STAT_HEIGHT);
       /* now the face */
       XSetStipple(dpy[disp], gc_addon[disp], pix_R);
       XSetClipMask(dpy[disp], gc_addon[disp], pix_G);
       XSetForeground(dpy[disp], gc_addon[disp], col[j].face );
       XFillRectangle(dpy[disp], pix_score[disp][SBDead + i*MAX_PLAYER +j],
-		     gc_addon[disp], 0, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
+		     gc_addon[disp], 0, 0, STAT_WIDTH, STAT_HEIGHT);
       /* reset gc to no clip mask */
       XSetClipMask(dpy[disp], gc_addon[disp], None);
     }
     XFreePixmap(dpy[disp],pix_R);
     XFreePixmap(dpy[disp],pix_G);
   }    
-  /* init numbered score tiles */
-  for (i=0; i <= MAX_VICTORIES; i++) {
-    pix_score[disp][MAX_SCORE_TILES+i-1] =
-	create_pixmap(disp, &(score_tile[MAX_SCORE_TILES-1]), fg, bg);
-    /* add box and and frame */
-    XFillRectangle(dpy[disp], pix_score[disp][MAX_SCORE_TILES+i-1],
-		   gc_text_black[disp], 9, 9, 31, 31);
-    XDrawRectangle(dpy[disp], pix_score[disp][MAX_SCORE_TILES+i-1],
-		   gc_text_white[disp], 9, 9, 31, 31);
-    sprintf(LivesText,"%1d",(unsigned int) i);
-    XSetFont(dpy[disp], gc_text_white[disp], font_struct[disp][FF_Large]->fid);
-    XDrawString(dpy[disp], pix_score[disp][MAX_SCORE_TILES+i-1],
-		gc_text_white[disp],
-		25 - XTextWidth(font_struct[disp][FF_Large], 
-				LivesText,strlen(LivesText))/2, 34,
-		LivesText, strlen(LivesText));
-  }
 }
 
 
@@ -624,26 +620,14 @@ init_score_tiles_bw (disp)
 #endif
 {
   int i;
-  static char LivesText[10];
 
   /* init led tiles  */
   for (i=0; i<2; i++) {
     pix_leds[disp][i] = create_black_white_map(disp, &(score_led[i]) );
   }
   /* init normal score tiles */
-  for (i=0; i < MAX_SCORE_TILES -1 ; i++)
+  for (i=0; i < MAX_SCORE_TILES; i++) {
     pix_score[disp][i] = create_black_white_map(disp, &(score_tile[i]) );
-  /* init numbered score tiles */
-  for (i=0; i <= MAX_VICTORIES; i++) {
-    pix_score[disp][MAX_SCORE_TILES+i-1] =
-      create_black_white_map(disp, &(score_tile[MAX_SCORE_TILES-1]) );
-    sprintf(LivesText,"%1d",(unsigned int) i);
-    XSetFont(dpy[disp], gc_text_white[disp], font_struct[disp][FF_Large]->fid);
-    XDrawString(dpy[disp], pix_score[disp][MAX_SCORE_TILES+i-1],
-		gc_text_white[disp],
-		25 - XTextWidth(font_struct[disp][FF_Large],
-				LivesText,strlen(LivesText))/2, 34,
-		LivesText, strlen(LivesText));
   }
 }
 
@@ -654,14 +638,14 @@ init_score_tiles_bw (disp)
  */
 #ifdef __STDC__
 static void 
-init_window (int disp)
+init_window_normal (int disp)
 #else
 static void 
-init_window (disp)
+init_window_normal (disp)
      int disp;
 #endif
 {
-  static char window_title[MAX_PLAYER][80];
+  static char window_title[MAX_DISPLAY][80];
 
   XWindowAttributes xwa;
   XSetWindowAttributes xswa;
@@ -690,7 +674,7 @@ init_window (disp)
   xswa.event_mask = NORMAL_EVENT_MASK;
   xswa.background_pixel = fg;
   xswa.border_pixel = fg;
-  xswa.override_redirect = (Bool) override_from_database (disp);
+  xswa.override_redirect = False;
 
   /* Open the Window */
   win[disp]
@@ -769,6 +753,87 @@ init_window (disp)
 
 
 
+/* 
+ * local function : init_window_override
+ */
+#ifdef __STDC__
+static void 
+init_window_override (int disp)
+#else
+static void 
+init_window_override (disp)
+     int disp;
+#endif
+{
+  XWindowAttributes xwa;
+  XSetWindowAttributes xswa;
+  XEvent xev;
+  XGCValues xgcv;
+  int fg, bg;
+
+  fg = black_pixel[disp];
+  bg = white_pixel[disp];
+
+  /* Get Root Window Size */
+  if ( XGetWindowAttributes(dpy[disp], DefaultRootWindow(dpy[disp]), &xwa)
+       == 0 ) {
+    x_fatal(disp,"couldn't get root window size","");
+  }
+
+  winX[disp] =  0;
+  winY[disp] =  0;
+  winW[disp] =  PIXW;
+  winH[disp] =  PIXH + SCOREH;
+
+  /* Set Window Attributes */
+  xswa.event_mask = NORMAL_EVENT_MASK;
+  xswa.background_pixel = fg;
+  xswa.border_pixel = fg;
+  xswa.override_redirect = True;
+
+  /* Open the Window */
+  win[disp]
+    = XCreateWindow(dpy[disp], DefaultRootWindow(dpy[disp]),
+                    winX[disp],winY[disp], winW[disp],winH[disp], 0,
+                    def_depth[disp],
+                    InputOutput,
+                    DefaultVisual(dpy[disp], DefaultScreen(dpy[disp]) ),
+                    CWEventMask | CWBackPixel | CWBorderPixel 
+		    | CWOverrideRedirect,
+                    &xswa );
+
+  xgcv.foreground = fg;
+  xgcv.background = bg;
+  gc_window[disp] = XCreateGC(dpy[disp], win[disp], 
+			      GCForeground | GCBackground, &xgcv);
+
+  /* Set Cursor */
+  XDefineCursor(dpy[disp], win[disp],
+                XCreateFontCursor(dpy[disp], XC_trek) );
+
+  /* Map the Window */
+  XMapRaised(dpy[disp], win[disp]);
+
+  /* wait for an ExposeEvent */
+  do {
+    XNextEvent(dpy[disp], &xev);
+  } while (xev.type != Expose );
+
+  /* get actual window size */
+  if ( XGetWindowAttributes(dpy[disp], win [disp], &xwa) == 0) {
+    x_fatal(disp,"could not get window size","");
+  }
+
+  winW[disp] = xwa.width;
+  winH[disp] = xwa.height;
+
+  if ( (winW[disp] < PIXW) || (winH[disp] <PIXH) ) {
+    x_warning(disp,"display is to small","");
+  }
+}
+
+
+
 /* 
  * Local function : init_pixmap_color
  */
@@ -1003,10 +1068,17 @@ init_sprites_color (disp)
   }
 
   /* bomb sprites */
+  add=alloc_color_or_black(disp, display_color[disp].bomb);
   for (i = 0; i < MAX_BOMBS; i++) {
     /* mask */
     pix_bomb_mask[disp][i] = create_bitmap(disp, &(bomb_mask[i]));
     pix_bomb_bits[disp][i] = create_pixmap(disp, &(bomb_bits[i]),fg, bg);
+    tmp = create_bitmap(disp, &(bomb_addon[i]));
+    XSetStipple(dpy[disp], gc_addon[disp], tmp);
+    XSetForeground(dpy[disp], gc_addon[disp],add);
+    XFillRectangle(dpy[disp], pix_bomb_bits[disp][i], gc_addon[disp], 
+		   0, 0, BLOCK_WIDTH, BLOCK_HEIGHT);
+    XFreePixmap(dpy[disp],tmp);
   }
   
   /* alloc colors for explosions */
@@ -1111,7 +1183,7 @@ init_display (disp, display)
   fprintf(stderr, "Display is \"%s\"\n", display);
 #endif
   /* Check Player */
-  if ( (disp < 0) || (disp >= MAX_PLAYER) ) {
+  if ( (disp < 0) || (disp >= MAX_DISPLAY) ) {
     x_fatal(disp,"wrong display","");
   }
 
@@ -1129,8 +1201,22 @@ init_display (disp, display)
   /* set standard pixel values */
   white_pixel[disp] = WhitePixel(dpy[disp], DefaultScreen(dpy[disp]));
   black_pixel[disp] = BlackPixel(dpy[disp], DefaultScreen(dpy[disp]));
+  
 }
 
+#ifdef __STDC__
+void
+finish_display (int disp)
+#else
+void
+finish_display (disp)
+  int disp;
+#endif
+{
+  if (dpy[disp] != NULL) {
+    XCloseDisplay(dpy[disp]);
+  }
+}
 
 
 /*
@@ -1148,6 +1234,9 @@ init_graphics (disp)
   int visual_class;
   XVisualInfo visual_info;
 
+  /* set key tables */
+  set_game_keys(disp);
+
   /* get color mode from visual */
   colorMode[disp] = FALSE;
   if (color_mode_from_database(disp) && (1 != def_depth[disp]) ) {
@@ -1162,16 +1251,20 @@ init_graphics (disp)
   fprintf(stderr,"Display %d: color mode is %d\n",disp, colorMode[disp]);
 #endif
 
+  /* init window, either override or normal */
+  if (override_from_database(disp)) {
+    init_window_override(disp);
+  } else {
+    init_window_normal(disp);
+  }
   /* init graphics either color or black an white */
   if (colorMode[disp]) {
-    init_window(disp);
     init_player_colors(disp);
     init_pixmap_color(disp);
     init_fonts_color(disp);
     init_score_tiles_color(disp);
     init_sprites_color(disp);
   } else {
-    init_window(disp);
     init_pixmap_bw(disp);
     init_fonts_bw(disp);
     init_score_tiles_bw(disp);
@@ -1181,33 +1274,97 @@ init_graphics (disp)
 }
 
 
+/*
+ * local variables for key translations
+ */
+
+#define KEYCODE_MAX 255
+#define KEYCODE_MIN 8
+#define NUM_KEYCODE 256
+
+static KeyPressAction *key_table[MAX_DISPLAY];
+static KeySym code2sym[2][NUM_KEYCODE];
+
+/*
+ * public link_keysyms
+ */
+#ifdef __STDC__
+void
+link_keysyms (int disp,
+	      int nelem,
+	      KeyPressDefine *keydef)
+#else
+void
+link_keysyms (disp, nelem, keydef)
+     int disp;
+     int nelem;
+     KeyPressDefine *keydef;
+#endif
+{
+
+  int i;
+  KeySym keysym;
+  KeyCode keycode;
+  char **keyv;
+  int keyc;
+
+  /* fill simple translation table */
+  for (keycode = KEYCODE_MIN; (keycode<KEYCODE_MAX) && (keycode!=0); keycode++) {
+    code2sym[0][keycode] = XKeycodeToKeysym(dpy[disp], keycode, 0);
+    code2sym[1][keycode] = XKeycodeToKeysym(dpy[disp], keycode, 1);
+  }
+
+  /* alloc key table */
+  if (NULL == (key_table[disp] 
+	       = calloc (NUM_KEYCODE, sizeof(KeyPressAction) ) ) ) {
+    x_fatal(disp, "failed to alloc key table", "");
+  }
+
+  for (i=0; i<nelem; i++) {
+    keyv = split_string(keydef[i].keysym, &keyc);
+    for (--keyc; keyc >= 0; keyc--) {
+      if (NULL == keyv[keyc]) {
+	x_fatal(disp, "undefined keysymbol", "");
+      }
+      if (NoSymbol == (keysym = XStringToKeysym(keyv[keyc]))) {
+	x_warning(disp, "unknown keysymbol", keyv[keyc]);
+	continue;
+      }
+      /* set entries in table */
+      for (keycode=KEYCODE_MIN; (keycode<KEYCODE_MAX)&&(keycode!=0);keycode ++) {
+	if ( (keysym == code2sym[0][keycode]) 
+	    || (keysym == code2sym[1][keycode])) {
+	  key_table[disp][keycode].addr  = keydef[i].addr;
+	  key_table[disp][keycode].value = keydef[i].value;
+	}
+      }
+    }
+  }
+
+}
+
+
+
 /*
  * public function: check_event
  */
 #ifdef __STDC__
 void 
 check_event (int disp, 
-	     void (*expose_func)(), 
-	     void (*key_func)() )
+	     void (*expose_func)()) 
 #else
 void 
-check_event (disp, expose_func, key_func)
+check_event (disp, expose_func)
      int disp;
      void (*expose_func)();
-     void (*key_func)();
 #endif
 {
   XEvent xev;
   int num_events;
 
   /* first get number of events */
-#if 0
-  /* events queued after flush */
-  num_events = XPending(dpy[disp]);
-#else
   XSync (dpy[disp], FALSE);
   num_events = XEventsQueued(dpy[disp],QueuedAlready);
-#endif
 
   while (num_events !=0) {
     num_events --;
@@ -1218,11 +1375,17 @@ check_event (disp, expose_func, key_func)
     switch (xev.type) {
       /* windows is iconfied */
     case UnmapNotify:
+#ifdef DEBUG
+      fprintf(stderr, "Window unmapped\n");
+#endif
       iconified[disp] = TRUE;
       break;
       
       /* window is mapped again */
     case MapNotify:
+#ifdef DEBUG
+      fprintf(stderr, "Window mapped\n");
+#endif
       iconified[disp] = FALSE;
       break;
 
@@ -1232,9 +1395,28 @@ check_event (disp, expose_func, key_func)
       break;
       
     case KeyPress:
-      /* turn keycode into x key symbol, 0 - means no modifier mask */
-      (*key_func)(disp, XKeycodeToKeysym(dpy[disp], xev.xkey.keycode, 0));
-     
+      {
+	unsigned keycode;
+	KeyPressAction *ptr;
+	
+	keycode = xev.xkey.keycode;
+	ptr = &(key_table[disp][keycode]);
+	if (NULL != ptr->addr) {
+	  *(ptr->addr) = ptr->value;
+	} else {
+	  /* dirty quick hack to be changed soon */
+	  if (disp == 0) {
+	    if (XK_Escape == XLookupKeysym(&xev.xkey, 0)) {
+	      (*quit_function)();
+	    }
+	  }
+	  /* first boss key, works only with window manager */
+	  if (XK_BackSpace == XLookupKeysym(&xev.xkey, 0)) {
+	    XIconifyWindow(dpy[disp],win[disp],DefaultScreen(dpy[disp]));
+	  }
+	}
+      }
+
       break;
     }
   }
@@ -1432,18 +1614,72 @@ free_explosion_blocks (disp)
 }
 
 
+static XRectangle void_list[MAZE_W*MAZE_H];
+static XRectangle *void_last = void_list;
+static XRectangle block_list[MAX_BLOCK][MAZE_W*MAZE_H];
+static XRectangle *block_last[MAX_BLOCK] = {
+  block_list[0],
+  block_list[1],
+  block_list[2],
+  block_list[3],
+  block_list[4],
+  block_list[5],
+  block_list[6],
+  block_list[7],
+  block_list[8],
+  block_list[9],
+  block_list[10],
+};
 
-
-/* public function : draw_block */
+/* 
+ * public function : draw_block 
+ */
 #ifdef __STDC__
 void 
-draw_block (int disp, 
-	    int x, 
+draw_block (int x, 
 	    int y, 
 	    int block)
 #else
 void 
-draw_block (disp, x, y, block)
+draw_block (x, y, block)
+     int x, y, block;
+#endif
+{
+#ifdef DEBUG
+  if ( (x < 0) || (x >= MAZE_W) 
+      || (y < 0) || (y >= MAZE_H)
+      || (block<-1) || (block >= MAX_BLOCK) ) {
+    fprintf(stderr,"ERROR in draw_block\n");
+    fprintf(stderr,"x = %d, y = %d, block =%d\n",x,y,block);
+  }
+#endif
+
+  if (block >= 0) {
+    block_last[block]->x = x*BLOCK_WIDTH;
+    block_last[block]->y = y*BLOCK_HEIGHT;
+    block_last[block]->width = BLOCK_WIDTH;
+    block_last[block]->height = BLOCK_HEIGHT;
+
+    block_last[block] ++;
+  } else {
+    void_last->x = x*BLOCK_WIDTH;
+    void_last->y = y*BLOCK_HEIGHT;
+    void_last->width = BLOCK_WIDTH;
+    void_last->height = BLOCK_HEIGHT;
+
+    void_last ++;
+  }
+}
+
+
+#ifdef __STDC__
+void 
+draw_block_at (int disp, 
+	       int x, int y, 
+	       int block)
+#else
+void 
+draw_block_at (disp, x, y, block)
      int disp, x, y, block;
 #endif
 {
@@ -1468,18 +1704,70 @@ draw_block (disp, x, y, block)
   }
 }
 
+static XRectangle expl_list[MAX_EXPLOSION][MAZE_W*MAZE_H];
+static XRectangle *expl_last[MAX_EXPLOSION] = {
+  expl_list[0],
+  expl_list[1],
+  expl_list[2],
+  expl_list[3],
+  expl_list[4],
+  expl_list[5],
+  expl_list[6],
+  expl_list[7],
+  expl_list[8],
+  expl_list[9],
+  expl_list[10],
+  expl_list[11],
+  expl_list[12],
+  expl_list[13],
+  expl_list[14],
+  expl_list[15],
+};
 
-
-/* public function : draw_explosion */
+/* 
+ * public function : draw_explosion 
+ */
 #ifdef __STDC__
 void 
-draw_explosion(int disp, 
-	       int x,
-	       int y, 
-	       int block)
+draw_explosion (int x,
+		int y, 
+		int block)
 #else
 void 
-draw_explosion(disp, x, y, block)
+draw_explosion(x, y, block)
+     int x, y, block;
+#endif
+{
+#ifdef DEBUG
+  if (block < 0) {
+    fprintf(stderr,"DRAW EXPLOSION: Negative Block Number %d\n",block);
+  }
+  if (block >= MAX_EXPLOSION) {
+    fprintf(stderr,"DRAW EXPLOSION: Block Number %d to large\n",block);
+  }
+#endif
+
+    expl_last[block]->x = x*BLOCK_WIDTH;
+    expl_last[block]->y = y*BLOCK_HEIGHT;
+    expl_last[block]->width = BLOCK_WIDTH;
+    expl_last[block]->height = BLOCK_HEIGHT;
+
+    expl_last[block] ++;
+}
+
+#if 0
+/* 
+ * public function : draw_explosion_at 
+ */
+#ifdef __STDC__
+void 
+draw_explosion_at (int disp, 
+		   int x,
+		   int y, 
+		   int block)
+#else
+void 
+draw_explosion_at (disp, x, y, block)
      int disp, x, y, block;
 #endif
 {
@@ -1497,7 +1785,54 @@ draw_explosion(disp, x, y, block)
                  (int)(BLOCK_WIDTH*x), (int)(BLOCK_HEIGHT*y),
                  BLOCK_WIDTH, BLOCK_HEIGHT);
 }
+#endif
 
+/* public function flush_blocks */
+#ifdef __STDC__
+void
+flush_blocks (int disp, 
+	      int flag)
+#else
+void
+flush_blocks (disp, flag)
+     int disp, flag;
+#endif
+{
+  int i;
+
+  /* void blocks */
+  if (void_last != void_list) {
+    XFillRectangles(dpy[disp], pix[disp], gc_clearpix[disp],
+		    void_list, void_last - void_list);
+    if (flag) {
+      void_last = void_list;
+    }
+  }
+
+  /* normal blocks */
+  for (i=0; i<MAX_BLOCK; i++) {
+    if (block_last[i] != block_list[i]) {
+      XSetTile(dpy[disp], gc_drawblock[disp], pix_block[disp][i]);
+      XFillRectangles(dpy[disp], pix[disp], gc_drawblock[disp],
+		      block_list[i], block_last[i] - block_list[i]);
+      if (flag) {
+	block_last[i] = block_list[i];
+      }
+    }
+  }
+
+  /* explosion blocks */
+  for (i=0; i<MAX_EXPLOSION; i++) {
+    if (expl_last[i] != expl_list[i]) {
+      XSetTile(dpy[disp], gc_drawblock[disp], pix_expl_block[disp][i]);
+      XFillRectangles(dpy[disp], pix[disp], gc_drawblock[disp],
+		      expl_list[i], expl_last[i] - expl_list[i]);
+      if (flag) {
+	expl_last[i] = expl_list[i];
+      }
+    }
+  }
+}
 
 
 /* public function : draw_explosion */
@@ -1544,12 +1879,11 @@ draw_explosion_sprite  (disp, x, y, block)
 /* local function flush_sync */
 #ifdef __STDC__
 static void 
-flush_sync (int disp, 
-	    int num_disp)
+flush_sync (int disp)
 #else
 static void 
-flush_sync(disp, num_disp)
-     int disp, num_disp;
+flush_sync(disp)
+     int disp;
 #endif
 {
 #ifdef hpux
@@ -1628,17 +1962,16 @@ init_fade(step)
 /* public function fade_out_window */
 #ifdef __STDC__
 void 
-fade_out_window (int disp, 
-		 int num_disp)
+fade_out_window (int disp)
 #else
 void 
-fade_out_window(disp, num_disp)
-     int disp, num_disp;
+fade_out_window (disp)
+     int disp;
 #endif
 {
   XDrawSegments(dpy[disp], win[disp], gc_window[disp],
                 line, fade_count);
-  flush_sync(disp, num_disp);
+  flush_sync(disp);
 }
 
 
@@ -1646,17 +1979,16 @@ fade_out_window(disp, num_disp)
 /* public function fade_in_window */
 #ifdef __STDC__
 void 
-fade_in_window(int disp, 
-		    int num_disp)
+fade_in_window (int disp)
 #else
 void 
-fade_in_window(disp, num_disp)
-     int disp, num_disp;
+fade_in_window (disp)
+     int disp;
 #endif
 {
   XDrawSegments(dpy[disp], win[disp], gc_frompix[disp],
                 line, fade_count);
-  flush_sync(disp, num_disp);
+  flush_sync(disp);
 }
 
 
@@ -1675,28 +2007,67 @@ clear_pixmap (disp)
 }
 
 
-
 /* public fucntion add_rectangle */
 #ifdef __STDC__
 void 
-add_rectangle (int x,
-	       int y)
+add_maze_rectangle (int x,
+		    int y)
 #else
 void 
-add_rectangle (x,y)
+add_maze_rectangle (x,y)
      int x,y;
 #endif
 {
+  xrec_max->height = BLOCK_HEIGHT;
+  xrec_max->x      = x*BLOCK_WIDTH;
+  xrec_max->y      = y*BLOCK_HEIGHT;
+  xrec_max->width  = BLOCK_WIDTH;
 
-  if (y < (MAZE_H+1) ) {
-    xrec_max->height = BLOCK_HEIGHT;
+  if (xrec_max != xrec) {
+    XRectangle *prev = xrec_max - 1;
+    
+    if ( (prev->y == xrec_max->y) 
+	&& ((xrec_max->x - prev->x) == prev->width) ) {
+      prev->width += BLOCK_WIDTH;
+      xrec_max = prev;
+    }
+  }
+
+  xrec_max ++;
+}
+
+
+
+#ifdef __STDC__
+void 
+add_stat_rectangle (int x,
+		    int y)
+#else
+void 
+add_stat_rectangle (x,y)
+     int x,y;
+#endif
+{
+  if (y == 0) {
+    xrec_max->height = STAT_HEIGHT;
   } else {
     xrec_max->height = LED_HEIGHT;
   }
 
-  xrec_max->x      = x*BLOCK_WIDTH;
-  xrec_max->y      = y*BLOCK_HEIGHT;
-  xrec_max->width  = BLOCK_WIDTH;
+  xrec_max->x      = x*STAT_WIDTH;
+  xrec_max->y      = MAZE_H*BLOCK_HEIGHT + y*STAT_HEIGHT;
+  xrec_max->width  = STAT_WIDTH;
+
+  if (xrec_max != xrec) {
+    XRectangle *prev = xrec_max - 1;
+    
+    if ( (prev->y == xrec_max->y) 
+	&& ((xrec_max->x - prev->x) == prev->width) ) {
+      prev->width += BLOCK_WIDTH;
+      xrec_max = prev;
+    }
+  }
+
 
   xrec_max ++;
 }
@@ -1706,18 +2077,16 @@ add_rectangle (x,y)
 /* public function flush_score_board */
 #ifdef __STDC__
 void 
-flush_score_board (int disp, 
-		   int num_disp)
+flush_score_board (int disp)
 #else
 void 
-flush_score_board (disp, num_disp)
+flush_score_board (disp)
      int disp;
-     int num_disp;
 #endif
 {
   XFillRectangle( dpy[disp], win[disp], gc_frompix[disp],
                  0, PIXH, PIXW, SCOREH);
-  flush_sync(disp, num_disp);
+  flush_sync(disp);
 }
 
 
@@ -1739,245 +2108,127 @@ flush_pixmap (disp, num_disp, flag)
     XFillRectangle( dpy[disp], win[disp], gc_frompix[disp],
                    0, 0, PIXW, PIXH + SCOREH );
   } else {
-#if 0
+#ifdef CLEAR_WINDOW
     XClearWindow( dpy[disp], win[disp] );
 #endif 
     if (!iconified[disp]) {
-      XFillRectangles( dpy[disp], win[disp], gc_frompix[disp],
+      XFillRectangles( dpy[disp], win[disp], gc_frompix[disp], 
 		      xrec, xrec_max - xrec );
     }
     if (disp == (num_disp -1)) {
       xrec_max = xrec;
     }
   }
-  flush_sync(disp, num_disp);
+  flush_sync(disp);
 }
 
-/* public function draw_sprites */
+/*
+ * public function win_is_mapped
+ */
 #ifdef __STDC__
-void 
-draw_sprites (int disp)
+int
+win_is_mapped (int disp)
 #else
-void 
-draw_sprites (disp)
+int
+win_is_mapped (disp)
      int disp;
 #endif
 {
-  Sprite *spl;
+  return !iconified[disp];
+}
 
-  if (!iconified[disp]) {
-    spl = sprite_max;
-    while  (spl != sprite_list) {
-      spl --;
-      
-      /* draw sprite as clipped tile when in color mode */
-      if ( (spl->any.mode) && colorMode[disp] ) {
-	
-	/* set drawing offset */
-	XSetClipOrigin(dpy[disp], gc_sprite_bits[disp], 
-		       spl->any.x, spl->any.y);
-	XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], 
-		     spl->any.x, spl->any.y);
-	
-	switch(spl->type) {
-	  
-	case STPlayer:
-	  XSetClipMask(dpy[disp], gc_sprite_bits[disp], 
-		       pix_sprite_mask[disp][spl->player.anime] );
-	  XSetTile(dpy[disp], gc_sprite_bits[disp], 
-		   pix_sprite_bits[disp][spl->player.player]
-		   [spl->player.anime]);
-	  XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
-			 (int)spl->any.x, (int)spl->any.y,
-			 (spl->player.anime == WINNER_ANIME) ? 
-			 WINNER_WIDTH : SPRITE_WIDTH, 
-			 (spl->player.anime == WINNER_ANIME) ? 
-			 WINNER_HEIGHT : SPRITE_HEIGHT);
-	  break;
-	  
-	case STBomb:
-	  XSetClipMask(dpy[disp], gc_sprite_bits[disp], 
-		       pix_bomb_mask[disp][spl->expl.anime] );
-	  XSetTile(dpy[disp], gc_sprite_bits[disp], 
-		   pix_bomb_bits[disp][spl->expl.anime] );
-	  XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
-			 (int)spl->any.x, (int)spl->any.y,
-			 BLOCK_WIDTH, BLOCK_HEIGHT);
-	  
-	}
-      } else {
-	/* draw sprite mask if not in color mode or mask is needed*/
-	
-	XSetTSOrigin(dpy[disp], gc_sprite_mask[disp],
-		     spl->any.x, spl->any.y);
-	switch(spl->type) {
-	case STPlayer:
-	  XSetStipple(dpy[disp], gc_sprite_mask[disp],
-		      pix_sprite_mask[disp][spl->player.anime] );
-	  XFillRectangle(dpy[disp], pix[disp], gc_sprite_mask[disp],
-			 (int)spl->any.x, (int)spl->any.y,
-			 (spl->player.anime == WINNER_ANIME) ? 
-			 WINNER_WIDTH : SPRITE_WIDTH, 
-			 (spl->player.anime == WINNER_ANIME) ? 
-			 WINNER_HEIGHT : SPRITE_HEIGHT);
-	  break;
-	  
-	case STBomb:
-	  XSetStipple(dpy[disp], gc_sprite_mask[disp],
-		      pix_bomb_mask[disp][spl->expl.anime] );
-	  XFillRectangle(dpy[disp], pix[disp], gc_sprite_mask[disp],
-			 (int)spl->any.x, (int)spl->any.y,
-			 BLOCK_WIDTH, BLOCK_HEIGHT);
-	  break;
-	}
-      }
-      /* draw sprite as stipple when in monochrome mode */
-      if (!colorMode[disp] && (spl->any.mode) ) {
-	/* set drawing offset */
-	XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], 
-		     spl->any.x, spl->any.y);
-	
-	switch(spl->type) {
-	case STPlayer:
-	    XSetStipple(dpy[disp], gc_sprite_bits[disp], 
-			pix_sprite_bits[disp][spl->player.player]
-			[spl->player.anime]);
-	    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
-			   (int)spl->any.x, (int)spl->any.y,
-			   (spl->player.anime == WINNER_ANIME) ? 
-			   WINNER_WIDTH : SPRITE_WIDTH, 
-			   (spl->player.anime == WINNER_ANIME) ? 
-			   WINNER_HEIGHT : SPRITE_HEIGHT);
-	    break;
-	    
-	  case STBomb:
-	    XSetStipple(dpy[disp], gc_sprite_bits[disp], 
-			pix_bomb_bits[disp][spl->expl.anime] );
-	    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
-			   (int)spl->any.x, (int)spl->any.y,
-			   BLOCK_WIDTH, BLOCK_HEIGHT);
-	      
-	  }
-      }
-    }
+/* public function draw_bomb_sprite */
+#ifdef __STDC__
+void 
+draw_bomb_sprite (int disp, 
+		  Sprite *ptr)
+#else
+void 
+draw_bomb_sprite (disp, ptr)
+     int disp;
+     Sprite *ptr;
+#endif
+{
+  BombSprite *spl = (BombSprite *)ptr;
+
+  /* draw sprite as clipped tile when in color mode */
+  if ( (spl->mode) && colorMode[disp] ) {
+    /* set drawing offset */
+    XSetClipOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetClipMask(dpy[disp], gc_sprite_bits[disp], 
+		 pix_bomb_mask[disp][spl->anime] );
+    XSetTile(dpy[disp], gc_sprite_bits[disp],
+	     pix_bomb_bits[disp][spl->anime] );
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
+		   (int)spl->x, (int)spl->y, BLOCK_WIDTH, BLOCK_HEIGHT);
+  } else {
+    /* draw sprite mask if not in color mode or mask is needed*/
+    XSetTSOrigin(dpy[disp], gc_sprite_mask[disp], spl->x, spl->y);
+    XSetStipple(dpy[disp], gc_sprite_mask[disp],
+		pix_bomb_mask[disp][spl->anime] );
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_mask[disp],
+		   (int)spl->x, (int)spl->y, BLOCK_WIDTH, BLOCK_HEIGHT);
+  }
+  /* draw sprite as stipple when in monochrome mode */
+  if (!colorMode[disp] && (spl->mode) ) {
+    /* set drawing offset */
+    XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetStipple(dpy[disp], gc_sprite_bits[disp], 
+		pix_bomb_bits[disp][spl->anime] );
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
+		   (int)spl->x, (int)spl->y, BLOCK_WIDTH, BLOCK_HEIGHT);
+    
   }
 }
 
 
-
-/* public function add_to_sprite_list */
-
+/* public function draw_player_sprite */
 #ifdef __STDC__
 void 
-add_trophy_to_sprite_list (int x,
-			   int y)
+draw_player_sprite (int disp, Sprite *ptr)
 #else
 void 
-add_trophy_to_sprite_list (x, y)
-     int x, y;
+draw_player_sprite (disp, ptr)
+     int disp;
+     Sprite *ptr;
 #endif
 {
-  sprite_max->type = STBomb;
-  sprite_max->expl.ysort = (y*2+4) * BLOCK_HEIGHT + 6 + 24;
-  sprite_max->expl.x = (x+6) * BLOCK_WIDTH;
-  sprite_max->expl.y = (y*2+4) * BLOCK_HEIGHT + 24;
-  sprite_max->expl.anime = BB_NORMAL;
-  sprite_max->expl.mode = TRUE;
-  sprite_max ++;
-}
-
-
-
-
-/* public function add_bomb_to_sprite_list */
-#ifdef __STDC__
-void 
-add_bomb_to_sprite_list (int x, 
-			 int y, 
-			 int an, 
-			 int mode)
-#else
-void 
-add_bomb_to_sprite_list (x, y, an, mode)
-     int x, y, an, mode;
-#endif
-{
-  sprite_max->type = STBomb;
-  sprite_max->expl.ysort = y + 6 ;
-  sprite_max->expl.x = x ;
-  sprite_max->expl.y = y ;
-  sprite_max->expl.anime = an;
-  sprite_max->expl.mode = mode;
-  sprite_max ++;
-}
-
-
-
-/* public function add_to_sprite_list */
-#ifdef __STDC__
-void 
-add_player_to_sprite_list (int pl, 
-			   int x, 
-			   int y, 
-			   int an, 
-			   int m)
-#else
-void 
-add_player_to_sprite_list (pl, x, y, an, m)
-     int pl;
-     int x, y, an;
-     int m;
-#endif
-{
-  sprite_max->type = STPlayer;
-  sprite_max->player.player = pl;
-  sprite_max->player.ysort = y + BLOCK_HEIGHT;
-  sprite_max->player.x = 
-    x + ( (an==WINNER_ANIME) ? WINNER_X_OFF : SPRITE_X_OFF );
-  sprite_max->player.y = 
-    y +( (an==WINNER_ANIME) ? WINNER_Y_OFF : SPRITE_Y_OFF );
-  sprite_max->player.anime = an;
-  sprite_max->player.mode = m;
-  sprite_max ++;
-}
-
-
-
-/* public function clear_sprite_list */
-#ifdef __STDC__
-void 
-clear_sprite_list (void)
-#else
-void 
-clear_sprite_list ()
-#endif
-{
-  sprite_max = sprite_list;
-}
-
-
-
-/* public sort_sprite_list */
-
-#ifdef __STDC__
-void 
-sort_sprite_list (void)
-#else
-void 
-sort_sprite_list ()
-#endif
-{
-  static Sprite swap;
-  Sprite *sp1,*sp2;
-
-  for (sp1 = sprite_list ; sp1 < sprite_max; sp1++) {
-    for (sp2 = sp1 + 1 ; sp2 < sprite_max; sp2++) {
-      if (sp1->any.ysort < sp2->any.ysort) {
-	swap = *sp1;
-	*sp1 = *sp2;
-	*sp2 = swap;
-      }
-    }
+  PlayerSprite *spl = (PlayerSprite *)ptr;
+  
+  /* draw sprite as clipped tile when in color mode */
+  if ( (spl->mode) && colorMode[disp] ) {
+    /* set drawing offset */
+    XSetClipOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetClipMask(dpy[disp], gc_sprite_bits[disp], 
+		 pix_sprite_mask[disp][spl->anime] );
+    XSetTile(dpy[disp], gc_sprite_bits[disp], 
+	     pix_sprite_bits[disp][spl->player][spl->anime]);
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
+		   (int)spl->x, (int)spl->y,
+		   (spl->anime == WINNER_ANIME) ? WINNER_WIDTH :SPRITE_WIDTH, 
+		   (spl->anime == WINNER_ANIME) ? WINNER_HEIGHT:SPRITE_HEIGHT);
+  } else {
+    /* draw sprite mask if not in color mode or mask is needed*/
+    XSetTSOrigin(dpy[disp], gc_sprite_mask[disp], spl->x, spl->y);
+    XSetStipple(dpy[disp], gc_sprite_mask[disp],
+		pix_sprite_mask[disp][spl->anime] );
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_mask[disp],
+		   (int)spl->x, (int)spl->y,
+		   (spl->anime == WINNER_ANIME) ? WINNER_WIDTH:SPRITE_WIDTH, 
+		   (spl->anime == WINNER_ANIME) ? WINNER_HEIGHT:SPRITE_HEIGHT);
+  }
+  /* draw sprite as stipple when in monochrome mode */
+  if (!colorMode[disp] && (spl->mode) ) {
+    /* set drawing offset */
+    XSetTSOrigin(dpy[disp], gc_sprite_bits[disp], spl->x, spl->y);
+    XSetStipple(dpy[disp], gc_sprite_bits[disp], 
+		pix_sprite_bits[disp][spl->player][spl->anime]);
+    XFillRectangle(dpy[disp], pix[disp], gc_sprite_bits[disp],
+		   (int)spl->x, (int)spl->y,
+		   (spl->anime == WINNER_ANIME) ? WINNER_WIDTH :SPRITE_WIDTH, 
+		   (spl->anime == WINNER_ANIME) ? WINNER_HEIGHT:SPRITE_HEIGHT);
   }
 }
 
@@ -1997,7 +2248,8 @@ draw_time_led (disp, x, block)
 {
   XSetTile(dpy[disp], gc_drawblock[disp], pix_leds[disp][block]);
   XFillRectangle(dpy[disp], pix[disp], gc_drawblock[disp],
-                 x*LED_WIDTH, (MAZE_H+1)*BLOCK_HEIGHT, LED_WIDTH, LED_HEIGHT);
+                 x*LED_WIDTH, MAZE_H*BLOCK_HEIGHT + STAT_HEIGHT, 
+		 LED_WIDTH, LED_HEIGHT);
 }
 
 
@@ -2016,8 +2268,35 @@ draw_score_block (disp, x, block)
 {
   XSetTile(dpy[disp], gc_drawblock[disp], pix_score[disp][block]);
   XFillRectangle(dpy[disp], pix[disp], gc_drawblock[disp],
-		 x*BLOCK_WIDTH, MAZE_H*BLOCK_HEIGHT,
-		 BLOCK_WIDTH, BLOCK_HEIGHT );
+		 x*STAT_WIDTH, MAZE_H*BLOCK_HEIGHT,
+		 STAT_WIDTH, STAT_HEIGHT );
+}
+
+
+/* public function draw_score_block */
+#ifdef __STDC__
+void 
+draw_score_block_half (int disp, 
+		       int x, 
+		       int block,
+		       int left)
+		       
+#else
+void 
+draw_score_block_half (disp, x, block, left)
+     int disp, x, block, left;
+#endif
+{
+  XSetTile(dpy[disp], gc_drawblock[disp], pix_score[disp][block]);
+  if (left) {
+    XFillRectangle(dpy[disp], pix[disp], gc_drawblock[disp],
+		   x*STAT_WIDTH, MAZE_H*BLOCK_HEIGHT,
+		   7*STAT_WIDTH/12, STAT_HEIGHT );
+  } else {
+    XFillRectangle(dpy[disp], pix[disp], gc_drawblock[disp],
+		   (12*x+7)*STAT_WIDTH/12, MAZE_H*BLOCK_HEIGHT,
+		   5*STAT_WIDTH/12, STAT_HEIGHT );
+  }
 }
 
 
@@ -2090,6 +2369,7 @@ draw_textbox (disp, text, flags, rect)
   XFontStruct *font;
   int y, width, height;
   GC gc_fg, gc_bg;
+  XRectangle clip;
 
   /* first get used font */
   font = font_struct[disp][FM_Size & flags];
@@ -2128,6 +2408,12 @@ draw_textbox (disp, text, flags, rect)
 
   /* draw string */
   if (NULL != text) {
+    /* set clipping rectangles */
+    clip.x = rect->x;
+    clip.y = rect->y;
+    clip.width = rect->w;
+    clip.height = rect->h;
+    XSetClipRectangles(dpy[disp], gc_fg, 0, 0, &clip, 1, YXBanded);
     /* dimensions of text */
     width = XTextWidth(font, text, strlen(text) );
     height = font->max_bounds.ascent - font->max_bounds.descent;
@@ -2136,6 +2422,8 @@ draw_textbox (disp, text, flags, rect)
     XDrawString(dpy[disp], pix[disp], gc_fg, 
 		rect->x + (rect->w-width)/2, rect->y + (height+rect->h)/2,
 		text, strlen(text));
+    /* reset clip mask */
+    XSetClipMask(dpy[disp], gc_fg, None);
   }
 }
 
