@@ -1,7 +1,7 @@
 /*
- * Programm XBLAST V1.2.6 or higher
+ * Programm XBLAST V2.1.10 or higher
  * (C) by Oliver Vogel (e-mail: vogel@ikp.uni-koeln.de)
- * February 6th 1996
+ * November 5th 1996
  * started August 1993
  *
  * File: intro.c
@@ -13,7 +13,7 @@
  * by the Free Software Foundation; either version 2; or (at your option)
  * any later version
  *
- * This program is distributed in the hope that it will entertaining,
+ * This program is distributed in the hope that it will be entertaining,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILTY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
  * Publis License for more details.
@@ -40,17 +40,21 @@
 #include "data.h"
 #include "info.h"
 #include "maze.h"
+#include "map.h"
 #include "main.h"
 #include "event.h"
 #include "status.h"
 #include "intro.h"
 #include "introdat.h"
+#include "pipe.h"
+
+#if defined(XBLAST_SOUND)
+#include "sound.h"
+#endif
 
 #define SCORE_TIME 16
 
-
-
-
+#define PARTNER(p) ( ((p)/2)*2 + 1 - ((p)%2) )
 
 /*
  * local function draw_intro_player
@@ -74,22 +78,36 @@ draw_intro_player (player, name, config)
   disp_name[0]='1'+config->pl_at_disp[player];
   add_player_to_sprite_list(player, intro_player_pos[player].x, 
 			    intro_player_pos[player].y, 0, TRUE);
+  if (config->team_mode == TM_Double) {
+    add_player_to_sprite_list(player, intro_player_pos[player].x, 
+			      intro_player_pos[player].y - BLOCK_HEIGHT, 
+			      0, TRUE);
+  }
+  /* Floor */
+  draw_block(control_pos[player][0].x, control_pos[player][0].y, 0);
+  draw_block(control_pos[player][0].x, control_pos[player][0].y-1, 0);
   for (disp = 0; disp < config->num_disp; disp ++) {
     draw_textbox(disp, name, FF_White|FF_Boxed|FF_Large, 
 		 intro_player_box+player);
-    /* Floor */
-    draw_block(disp, control_pos[player][0].x, control_pos[player][0].y, 0);
+    if (config->team_mode != TM_Double) {
+      /* alpha control */
+      if (player == disp_player[disp].p2) {
+	draw_block_at(disp,control_pos[player][2].x,control_pos[player][2].y, 2);
+      }
+      /* num control */
+      if (player == disp_player[disp].p1) {
+	draw_block_at(disp,control_pos[player][3].x,control_pos[player][3].y, 1);
+      }
+    } else {
+      /* both controls for double mode */
+      if (player == disp_player[disp].p1) {
+	draw_block_at(disp,control_pos[player][2].x,control_pos[player][2].y, 2);
+	draw_block_at(disp,control_pos[player][3].x,control_pos[player][3].y, 1);
+      }
+    }
     /* Display */
-    draw_block(disp, control_pos[player][1].x, control_pos[player][1].y, 3);
+    draw_block_at(disp, control_pos[player][1].x, control_pos[player][1].y, 3);
     draw_textbox(disp, disp_name, FF_Large | FF_Black, disp_box+player);
-    /* alpha control */
-    if (player == disp_player[disp].p2) {
-      draw_block(disp, control_pos[player][2].x, control_pos[player][2].y, 2);
-    }
-    /* num control */
-    if (player == disp_player[disp].p1) {
-      draw_block(disp, control_pos[player][3].x, control_pos[player][3].y, 1);
-    }
   }
 
   switch(player) {
@@ -138,6 +156,12 @@ do_intro (ps, p_string, config)
   int count;
   int player;
   int i, flags;
+  double pfactor;
+
+#if defined (XBLAST_SOUND)
+  load_sound(SND_WHIRL);
+#endif
+
 
   for (disp = 0; disp < config->num_disp; disp ++) {
     init_block(disp, BLIronFloor, BTFree, "Black","Gray75","MidnightBlue");
@@ -159,6 +183,10 @@ do_intro (ps, p_string, config)
 		 intro_box+2);
   }
   
+#if defined(XBLAST_SOUND)
+  play_sound(SND_WHIRL, SOUND_MIDDLE_POSITION);
+#endif
+
   set_fade_max(PIXH+SCOREH);
   fade_in(config->num_disp);
   
@@ -176,13 +204,17 @@ do_intro (ps, p_string, config)
 
   for (disp = 0; disp < config->num_disp; disp ++) {
     clear_pixmap(disp);
-    draw_polygon(disp, 5*BLOCK_WIDTH, BLOCK_HEIGHT, 
-		 BLOCK_WIDTH*5, BLOCK_HEIGHT*6, pointx, SIZE_OF_X, TRUE);
   }
 
+#if defined (XBLAST_SOUND)
+  stop_sound(STOP_ALL_SOUNDS);
+  unload_sound(SND_WHIRL);
+  load_sound(SND_INTRO);
+  load_sound(SND_EXPL);
+#endif
+
   /* update status bar */
-  init_status_bar(config->num_player, config->num_disp, ps, "Press Space", 
-		  FALSE);
+  init_status_bar(config, ps, "Press Space", FALSE);
 
   fade_in(config->num_disp);
 
@@ -194,6 +226,11 @@ do_intro (ps, p_string, config)
   init_timer();
   clear_sprite_list();
   for (count = 0; count < INTRO_LENGTH; count ++) {
+#if defined(XBLAST_SOUND)
+    if (! (count % CHAR_ANIME) ) {
+      play_sound(SND_EXPL, (count/CHAR_ANIME)*3+1);
+    }
+#endif
     if (count < CHAR_ANIME) {
       copy_expl_block( 0, 8, Block_B[count]);
     } else if (count < (CHAR_ANIME*2)) {
@@ -205,7 +242,20 @@ do_intro (ps, p_string, config)
     } else {
       copy_expl_block(12, 8, Block_T[count-CHAR_ANIME*4]);
     }
-     
+
+    /* draw growing poly */
+    pfactor = (double)(count+1)/INTRO_LENGTH;
+    for (disp=0; disp < config->num_disp; disp ++) {
+      draw_polygon(disp, 
+		   (int)(0.5+(7.5-2.5*pfactor)*BLOCK_WIDTH), 
+		   (int)(0.5+(10.0-9.0*pfactor)*BLOCK_HEIGHT), 
+		   (int)(0.5+pfactor*5.0*BLOCK_WIDTH), 
+		   (int)(0.5+pfactor*6.0*BLOCK_HEIGHT), 
+		   pointx, SIZE_OF_X, TRUE);
+    }
+    mark_maze(5,1,9,10);
+
+
     if (count==(INTRO_LENGTH-1)) {
       for (disp = 0; disp < config->num_disp; disp ++) {
 	for (i=0; i<NUM_XC; i++) {
@@ -214,8 +264,26 @@ do_intro (ps, p_string, config)
       }
       mark_maze(4, 9, 10, 11 ); 
       mark_maze(0,0,14,12);
-      for (player = 0; player < config->num_player; player ++) {
-	draw_intro_player(player, p_string[player].name, config);
+      /* draw players */
+      if (config->team_mode != TM_Double) {
+	for (player = 0; player < config->num_player; player ++) {
+	  draw_intro_player(player, p_string[player].tag, config);
+	}
+      } else {
+	for (player = 0; player < config->num_player/2; player ++) {
+	  draw_intro_player(player, p_string[player].tag, config);
+	}
+      }
+      /* draw team tags if needed */
+      if (config->team_mode == TM_Team) {
+	for (i = 0; i < (config->num_player / 2); i ++) {
+	  for (disp = 0; disp < config->num_disp; disp ++) {
+	    draw_textbox(disp, NULL, FF_Medium|FF_White|FF_Boxed,
+			 team_box2 + i );
+	    draw_textbox(disp, team_string[i], FF_Medium|FF_Black|FF_Boxed,
+			 team_box + i );
+	  }
+	}
       }
     }
     for (disp = 0; disp < config->num_disp; disp ++) {
@@ -223,6 +291,11 @@ do_intro (ps, p_string, config)
     } 
   }
 
+    /* nachher hier die fanfare */
+#if defined(XBLAST_SOUND)
+  play_sound(SND_INTRO, SOUND_MIDDLE_POSITION);
+#endif
+
   init_timer();
   clear_keys(config->num_player);
   do {
@@ -235,6 +308,11 @@ do_intro (ps, p_string, config)
   for (disp = 0; disp < config->num_disp; disp ++) {
     free_block(disp, BTFree);
   }
+#if defined(XBLAST_SOUND)
+  stop_sound(STOP_ALL_SOUNDS);
+  unload_sound(SND_INTRO);
+  unload_sound(SND_EXPL);
+#endif
 }
 
 
@@ -253,7 +331,7 @@ level_start(num_disp)
   int disp;
 
   for (disp =0; disp < num_disp; disp ++) {
-    flush_score_board(disp, num_disp);
+    flush_score_board(disp);
   }
   fade_out(num_disp);
 }
@@ -269,67 +347,44 @@ level_start(num_disp)
 void 
 level_intro (int lvl, 
 	     BMPlayer *player_stat,
-	     XBSettings *setup,
 	     XBConfig *config)
 #else
 void 
-level_intro (lvl, player_stat, setup, config)
+level_intro (lvl, player_stat, config)
      int lvl;
      BMPlayer *player_stat;
-     XBSettings *setup; 
      XBConfig *config;
 #endif
 {
   int player;
   int disp;
   int i, count;
-  int min_range, min_bombs;
 
-  static char **extra_info = NULL;
-  static char **level_info = NULL;
-  static char **player_info = NULL;
-  
-  /* alloc info structures if needed */
-  if ( (NULL == extra_info) && (NULL == (extra_info = alloc_info()) ) ) {
-    fprintf(stderr, "Failed to alloc info structure\n");
-    exit(1);
-  }
-  if ( (NULL == level_info) && (NULL == (level_info = alloc_info()) ) ) {
-    fprintf(stderr, "Failed to alloc info structure\n");
-    exit(1);
-  }
-  if ( (NULL == player_info) && (NULL == (player_info = alloc_info()) ) ) {
-    fprintf(stderr, "Failed to alloc info structure\n");
-    exit(1);
-  }
+  char **extra_info;
+  char **level_info;
+  char **player_info;
 
-  /* Load maze and display initial positions */
-  load_maze(player_stat, lvl, config, setup, &min_range, &min_bombs);
-
-  for (player = 0; player < config->num_disp; player ++) {
-    draw_maze(player);
-  }
   reset_status_bar(player_stat, "Press Space", FALSE);
+  draw_maze(config->num_disp);
   
   /* draw player positions */
   for (player = 0; player < config->num_player; player ++) {
-    print_player_lives(player, FALSE);
     player_stat[player].anime = 0;
-    add_player_to_sprite_list(player,
+    add_player_to_sprite_list(player_stat[player].sprite,
 			      player_stat[player].x, player_stat[player].y,
 			      player_stat[player].anime, TRUE);
   }
 
-  for (player = 0; player < config->num_disp; player ++)
-    draw_sprites(player);
+  sort_sprite_list();
+
+  for (disp = 0; disp < config->num_disp; disp ++) {
+    draw_sprites(disp);
+  }
 
   clear_sprite_list();
 
   /* Display level info */
-
-  get_extra_info(lvl, extra_info);
-  get_level_info(lvl, level_info);
-  get_player_info(lvl, player_info);
+  get_info(&extra_info, &level_info, &player_info);
 
   for (disp = 0; disp < config->num_disp; disp ++) {
     /* draw level title */
@@ -374,8 +429,8 @@ level_intro (lvl, player_stat, setup, config)
 
   level_start (config->num_disp);
 
-  for (player = 0; player < config->num_disp; player ++) {
-    flush_pixmap(player, config->num_disp, FALSE);
+  for (disp = 0; disp < config->num_disp; disp ++) {
+    flush_pixmap(disp, config->num_disp, FALSE);
   }
 
   init_timer();
@@ -407,14 +462,14 @@ welcome (num_player, st)
   char *tmpl;
   int j;
   int p1,p2;
-  char *welcomes[MAX_PLAYER];
+  char *welcomes[2*MAX_PLAYER];
 
   for(p1=0; p1<num_player; p1++) {
     welcomes[p1] = st[p1].welcome;
   }
   for(j=0; j<3; j++) {
     for(p1=0; p1<num_player; p1++) {
-      p2 = rand() % num_player;
+      p2 = random_number(num_player);
       tmpl = welcomes[p1];
       welcomes[p1] = welcomes[p2];
       welcomes[p2] = tmpl;
@@ -432,53 +487,74 @@ welcome (num_player, st)
  */
 #ifdef __STDC__
 void 
-status_board (int last_player,
+status_board (int last_team,
 	      int num_victories,
 	      BMPlayer *player_stat,
 	      PlayerStrings *p_string,
-	      XBSettings *setup,
 	      XBConfig *config)
 #else
 void 
-status_board (last_player, num_victories, player_stat, p_string, setup, config)
-     int last_player;
+status_board (last_team, num_victories, player_stat, p_string, config)
+     int last_team;
      int num_victories;
      BMPlayer *player_stat;
      PlayerStrings *p_string;
-     XBSettings *setup;
      XBConfig *config;
 #endif
 {
   int player,disp , i;
   int count;
-  int min_range, min_bombs;
+  int box_set = 0;
+  int flag = 0;
 
   /* load score board maze */
-  load_maze(player_stat, STATUS_BOARD, config, setup, &min_range, &min_bombs);
+  load_maze(scoreBoard, config);
 
   /* draw maze in pixmap */
-  for (player = 0; player < config->num_disp; player ++) {
-    draw_maze(player);
-  }
-  reset_status_bar(player_stat, get_level_name(STATUS_BOARD), FALSE);
+  reset_status_bar(player_stat, get_level_name(scoreBoard), FALSE);
+  draw_maze(config->num_disp);
 
   /* draw audience */
-  for (i = 0; i < (3 * PIXW-128); i += 128) {
-    add_player_to_sprite_list( (rand()>>4)%MAX_PLAYER, i/3, 32, 0, TRUE);
+  if ( (config->num_player != 6) || (config->team_mode != TM_Single) ) {
+    for (i = 0; i < (3 * PIXW-128); i += 128) {
+      add_player_to_sprite_list( random_number(MAX_PLAYER), i/3, 36, 0, TRUE);
+    }
+    for (i = 64; i < (3 * PIXW-128); i += 128) {
+      add_player_to_sprite_list( random_number(MAX_PLAYER), i/3, -12, 0, TRUE);
+    }
   }
-  for (i = 64; i < (3 * PIXW-128); i += 128) {
-    add_player_to_sprite_list( (rand()>>4)%MAX_PLAYER, i/3, -16, 0, TRUE);
+  for (i = 0; i < (3 * PIXW-128); i += 128) {
+    add_player_to_sprite_list( random_number(MAX_PLAYER), i/3, -60, 0, TRUE);
   }
 
   /* draw player positions */
+  switch (config->team_mode) {
+  case TM_Single:
+    box_set = config->num_player-2;
+    flag = FF_Medium |FF_Boxed |FF_White;
+    break;
+
+  case TM_Team:
+    box_set = (config->num_player-4)/2+5;
+    flag = FF_Small |FF_Boxed |FF_White;
+    break;
+
+  case TM_Double:
+    box_set = (config->num_player/2)-2;
+    flag = FF_Medium |FF_Boxed |FF_White;
+    break;
+  }
+
   for (player = 0; player < config->num_player; player ++) {
-    print_player_lives(player, FALSE);
-    for (disp = 0; disp < config->num_disp; disp ++) {
-      draw_textbox(disp, p_string[player].name, FF_Medium |FF_Boxed |FF_White,
-		   score_player_box+player);
+    /* draw player names */
+    if ( (config->team_mode != TM_Double) || (player < config->num_player/2) ) {
+      for (disp = 0; disp < config->num_disp; disp ++) {
+	draw_textbox(disp, p_string[player].name, flag, 
+		     score_player_box[box_set]+player);
+      }
     }
-    
-    if (player == last_player) {
+    /* set player animes */
+    if (player_stat[player].team == last_team) {
       player_stat[player].anime = WINNER_ANIME;
     } else if (player_stat[player].victories == num_victories) {
       player_stat[player].anime = 0;
@@ -486,23 +562,23 @@ status_board (last_player, num_victories, player_stat, p_string, setup, config)
       player_stat[player].anime = 14;
     }
     
-    add_player_to_sprite_list(player,
+    add_player_to_sprite_list(player_stat[player].sprite,
 			      player_stat[player].x, player_stat[player].y,
 			      player_stat[player].anime, TRUE);
     for (i = 0; i < player_stat[player].victories; i ++) {
-      add_trophy_to_sprite_list(i, player);
+      add_trophy_to_sprite_list(i, player_stat[player].y+BLOCK_HEIGHT);
     }
   }
 
-  for (player = 0; player < config->num_disp; player ++) {
-    draw_sprites(player);
+  for (disp = 0; disp < config->num_disp; disp ++) {
+    draw_sprites(disp);
   }
   clear_sprite_list();
 
   level_start(config->num_disp);
 
-  for (player = 0; player < config->num_disp; player ++) {
-    flush_pixmap(player, config->num_disp, FALSE);
+  for (disp = 0; disp < config->num_disp; disp ++) {
+    flush_pixmap(disp, config->num_disp, FALSE);
   }
 
   init_timer();
@@ -523,15 +599,15 @@ status_board (last_player, num_victories, player_stat, p_string, setup, config)
 /* local function winning the game */
 #ifdef __STDC__
 void 
-winning_the_game (int last_player,
+winning_the_game (int last_team,
 		  BMPlayer *player_stat,
 		  PlayerStrings *p_string,
 		  XBSettings *setup,
 		  XBConfig *config)
 #else
 void 
-winning_the_game (last_player, player_stat, p_string, setup, config)
-     int last_player;
+winning_the_game (last_team, player_stat, p_string, setup, config)
+     int last_team;
      BMPlayer *player_stat;
      PlayerStrings *p_string;
      XBSettings *setup;
@@ -540,66 +616,104 @@ winning_the_game (last_player, player_stat, p_string, setup, config)
 
 {
   static char Message[80];
-  int disp,player;
-  int min_range, min_bombs;
+  int disp,player,pos;
   int i;
+  BMPlayer *ps, *last_ps;
+
+  last_ps = player_stat + last_team;
 
   for (disp = 0; disp < config->num_disp; disp++) {
     clear_window(disp);
   }
 
   /* load score board maze */
-  load_maze(player_stat, WINNING_THE_GAME, config, setup, 
-	    &min_range, &min_bombs);
+  load_maze(winningTheGame, config);
 
   /* draw maze in pixmap */
-  for (player = 0; player < config->num_disp; player ++) {
-    draw_maze(player);
-  }
   game_time = 0;
-  reset_status_bar(player_stat, get_level_name(WINNING_THE_GAME), TRUE);
+  reset_status_bar(player_stat, get_level_name(winningTheGame), FALSE);
+  draw_maze(config->num_disp);
 
   /* draw audience */
   for (i = 0; i < (3 * PIXW-128); i += 128) {
-    player = (rand()>>4)%MAX_PLAYER;
-    add_player_to_sprite_list(player, i/3, 32,
-			      (player == last_player)? 15 : 14 , TRUE);
+    player = random_number(MAX_PLAYER);
+    if (player_stat[player].team == last_team) {
+      add_player_to_sprite_list(player, i/3, 32, 15 , TRUE);
+    } else {
+      add_player_to_sprite_list(player, i/3, 32, 14 , TRUE);
+    }
   }
+
   for (i = 64; i < (3 * PIXW-128); i += 128) {
-    player = (rand()>>4)%MAX_PLAYER;
-    add_player_to_sprite_list(player, i/3, -16,
-			      (player == last_player)? 15 : 14 , TRUE);
+    player = random_number(MAX_PLAYER);
+    if (player_stat[player].team == last_team) {
+      add_player_to_sprite_list(player, i/3, -16, 15 , TRUE);
+    } else {
+      add_player_to_sprite_list(player, i/3, -16, 14 , TRUE);
+    }
+  }
+  for (i = 0; i < (3 * PIXW-128); i += 128) {
+    player = random_number(MAX_PLAYER);
+    if (player_stat[player].team == last_team) {
+      add_player_to_sprite_list(player, i/3, -64, 15 , TRUE);
+    } else {
+      add_player_to_sprite_list(player, i/3, -64, 14 , TRUE);
+    }
   }
 
   /* draw player positions */
-  for (player = 0; player < config->num_player; player ++)
-    {
-      print_player_lives(player, FALSE);
-      if (player != last_player) {
-	player_stat[player].anime = 14;
-	add_player_to_sprite_list(player,
-				  player_stat[player].x, 
-				  player_stat[player].y,
-				  player_stat[player].anime, TRUE);
-      }
+  pos = 0;
+  for (player = 0; player < config->num_player; player ++) {
+    if ( (config->team_mode == TM_Double) && (player == config->num_player/2) ) {
+      pos ++;
+    }
+    if (player_stat[player].victories != last_ps->victories) {
+      player_stat[player].anime = player_stat[player].victories ? 14 : 13;
+      add_player_to_sprite_list(player_stat[player].sprite,
+				player_stat[pos].x, player_stat[pos].y,
+				player_stat[player].anime, 
+				TRUE);
+      pos ++;
     }
-  
-  for (disp = 0; disp < config->num_disp; disp ++) {
-    draw_winner(disp, last_player,
-		(PIXW-BIG_WIDTH)/2, (PIXH-BIG_HEIGHT)/2-24 ); 
-    draw_textbox(disp, p_string[last_player].name,
-		 FF_White | FF_Medium | FF_Boxed, &winner_box);
   }
   
-  for (player = 0; player < config->num_disp; player ++) {
-    draw_sprites(player);
+  switch (config->team_mode) {
+  case TM_Single:
+    for (disp = 0; disp < config->num_disp; disp ++) {
+      draw_winner(disp, last_ps->sprite,
+		  (PIXW-BIG_WIDTH)/2, (PIXH-BIG_HEIGHT)/2-24 ); 
+      draw_textbox(disp, p_string[last_team].name,
+		   FF_White | FF_Medium | FF_Boxed, &winner_box);
+    }
+    break;
+
+  case TM_Team:
+  case TM_Double:
+      for (disp = 0; disp < config->num_disp; disp ++) {
+	int xpos = (PIXW-BIG_WIDTH)/2 - 3*BLOCK_WIDTH/2;
+	for (i=0, ps = player_stat; i<config->num_player; i++, ps++) {
+	  if (ps->team == last_team) {
+	    draw_winner(disp, ps->sprite, xpos, (PIXH-BIG_HEIGHT)/2-24 ); 
+	    xpos += 3*BLOCK_WIDTH;
+	  }
+	}
+	draw_textbox(disp, p_string[last_team].name,
+		     FF_White | FF_Medium | FF_Boxed, &two_winner_box);
+    }
+    break;
+
+  }
+  
+  for (disp = 0; disp < config->num_disp; disp ++) {
+    draw_sprites(disp);
   }
   clear_sprite_list();
 
   if (setup->print_stat) {
-    printf("GameStat {%s} {",p_string[last_player].name);
-    for (player =0; player < config->num_player; player ++)
+    printf("GameStat {%s} {",p_string[last_team].name);
+    for (player =0; player < config->num_player; player ++) {
       printf(" {%s} ",p_string[player].name);
+    }
     printf("}\n");
   }
 
@@ -609,9 +723,9 @@ winning_the_game (last_player, player_stat, p_string, setup, config)
     flush_pixmap(disp, config->num_disp, FALSE);
   }
   
-  sprintf(Message, p_string[last_player].wingame,
-	  p_string[last_player].name);
-  wait2_two_text(config->num_disp, config->num_player, Message,"Press Space");
+  sprintf(Message, p_string[last_team].wingame,  p_string[last_team].name);
+  wait2_two_text(config->num_disp, config->num_player, player_stat, Message,
+		 "Press Space");
   
   fade_out(config->num_disp);
 }
