@@ -1,7 +1,7 @@
 /*
- * Programm XBLAST V1.2.14
+ * Programm XBLAST V2.2
  * (C) by Oliver Vogel (e-mail: vogel@ikp.uni-koeln.de)
- * April 22nd 1996
+ * December 9th 1996
  * started August 1993
  *
  * File: patchlev.h
@@ -24,23 +24,33 @@
  */
 
 #ifdef _MAIN_C
-static char c_string[] = "XBlast 2.0 Copyright (c) 1993-96 Oliver Vogel" ;
+#ifdef XBLAST_SOUND
+static char c_string[] = "XBlast 2.2 Sound Copyright (c) 1993-96 Oliver Vogel";
+#else
+static char c_string[] = "XBlast 2.2 Copyright (c) 1993-96 Oliver Vogel";
+#endif
 #endif
 #ifdef _INTRO_C
+#ifdef XBLAST_SOUND
+#define NUM_XC 9
+#else
 #define NUM_XC 7
+#endif
 static char *xc_string[NUM_XC] = {
   NULL,
   NULL,
-  "XBlast  2.0",
+  "XBlast  2.2",
   "Copyright \251 1993-96 Oliver Vogel",
   "(vogel@ikp.uni-koeln.de)",
   "Coauthor Garth Denley",
   "(g0denley@teaching.cs.adelaide.edu.au)",
+#ifdef XBLAST_SOUND
+  "Sound by NorbertNicolay",
+  "(nicolay@ikp.uni-koeln.de)",
+#endif
 };
 #endif
 
 /*
  * end of file patchlev.h
  */
-
-
