@@ -1,7 +1,7 @@
 /*
- * Programm XBLAST V1.2.10 or higher
+ * Programm XBLAST V2.1.7 or higher
  * (C) by Oliver Vogel (e-mail: vogel@ikp.uni-koeln.de)
- * March 7th 1996
+ * September 1st 1996
  * started August 1993
  *
  * File: xreource.h 
@@ -26,12 +26,6 @@
 
 #ifndef _XRESOURCE_H
 #define _XRESOURCE_H
-/*
- * constants for getting single left, and right player of display
- */ 
-#define SINGLE_PLAYER  MAX_PLAYER
-#define RIGHT_PLAYER  (MAX_PLAYER+1)
-#define LEFT_PLAYER   (MAX_PLAYER+2)
 /*
  * resource string constants
  *
@@ -47,21 +41,29 @@
 #define N_AtDisplay2  "xblast.display2"
 #define N_AtDisplay3  "xblast.display3"
 #define N_AtDisplay4  "xblast.display4"
+#define N_AtDisplay5  "xblast.display5"
+#define N_AtDisplay6  "xblast.display6"
 static char *N_AtDisplay[MAX_PLAYER] = {
   N_AtDisplay1,
   N_AtDisplay2,
   N_AtDisplay3,
   N_AtDisplay4,
+  N_AtDisplay5,
+  N_AtDisplay6,
 };
 #define D_AtDisplay1  "XBlast.display1"
 #define D_AtDisplay2  "XBlast.display2"
 #define D_AtDisplay3  "XBlast.display3"
 #define D_AtDisplay4  "XBlast.display4"
+#define D_AtDisplay5  "XBlast.display5"
+#define D_AtDisplay6  "XBlast.display6"
 static char *D_AtDisplay[MAX_PLAYER] = {
   D_AtDisplay1,
   D_AtDisplay2,
   D_AtDisplay3,
   D_AtDisplay4,
+  D_AtDisplay5,
+  D_AtDisplay6,
 };
 #define C_AtDisplay "XBlast.Display"
 /* player names */
@@ -69,6 +71,8 @@ static char *D_AtDisplay[MAX_PLAYER] = {
 #define N_Player2 "xblast.player2"
 #define N_Player3 "xblast.player3"
 #define N_Player4 "xblast.player4"
+#define N_Player5 "xblast.player5"
+#define N_Player6 "xblast.player6"
 #define N_PlayerS "xblast.singlePlayer"
 #define N_PlayerR "xblast.rightPlayer"
 #define N_PlayerL "xblast.leftPlayer"
@@ -78,6 +82,8 @@ static char *N_Player[MAX_PLAYER+3] = {
   N_Player2,
   N_Player3,
   N_Player4,
+  N_Player5,
+  N_Player6,
   N_PlayerS,
   N_PlayerR,
   N_PlayerL,
@@ -86,17 +92,26 @@ static char *N_Player[MAX_PLAYER+3] = {
 #define D_Player2 "XBlast.player2"
 #define D_Player3 "XBlast.player3"
 #define D_Player4 "XBlast.player4"
+#define D_Player5 "XBlast.player5"
+#define D_Player6 "XBlast.player6"
 static char *D_Player[MAX_PLAYER] = {
   D_Player1,
   D_Player2,
   D_Player3,
   D_Player4,
+  D_Player5,
+  D_Player6,
 };
 #define C_Player "XBlast.Player"
 /* number of players */
 #define W_NumberOfPlayers ".numberOfPlayers"
 #define N_NumberOfPlayers "xblast.numberOfPlayers"
 #define C_NumberOfPlayers "XBlast.NumberOfPlayers"
+/* team mode */
+#define W_TeamMode        ".teamMode"
+#define D_TeamMode        "XBlast.teamMode"
+#define N_TeamMode        "xblast.teamMode"
+#define C_TeamMode        "XBlast.TeamMode"
 /* starting level */
 #define W_StartingLevel  ".startingLevel"
 #define D_StartingLevel  "XBlast.startingLevel"
@@ -135,6 +150,11 @@ static char *D_Player[MAX_PLAYER] = {
 #define D_ForceOverride "XBlast.forceOverride"
 #define N_ForceOverride "xblast.forceOverride"
 #define C_ForceOverride "XBlast.ForceOverride"
+/* fork mode */
+#define W_Fork ".fork"
+#define D_Fork "XBlast.fork"
+#define N_Fork "xblast.fork"
+#define C_Fork "XBlast.Fork"
 /* sound using bell */
 #define W_BellSound  ".bellSound"
 #define D_BellSound  "XBlast.bellSound"
@@ -176,6 +196,10 @@ static char *D_Player[MAX_PLAYER] = {
 #define W_PrintLevels  ".printLevels"
 #define N_PrintLevels  "xblast.printLevels"
 #define C_PrintLevels  "XBlast.PrintLevels"
+/* print a list of levels for tkXBlast */
+#define W_PrintLevelsTcl  ".printLevelsTcl"
+#define N_PrintLevelsTcl  "xblast.printLevelsTcl"
+#define C_PrintLevelsTcl  "XBlast.PrintLevelsTcl"
 
 /*
  * display resources
@@ -238,6 +262,10 @@ static char *D_Player[MAX_PLAYER] = {
 #define C_ExplosionColor1  "XBlast.ExplosionColor"
 #define C_ExplosionColor2  "XBlast.ExplosionColor"
 #define C_ExplosionColor3  "XBlast.ExplosionColor"
+/* color for bombs */
+#define N_BombColor "xblast.bombColor"
+#define C_BombColor "XBlast.BombColor"
+#define D_BombColor "XBlast.bombColor"
 
 /* 
  * player resources
@@ -250,12 +278,16 @@ static char *D_PlayerWinLevel[MAX_PLAYER] = {
   "XBlast.player2.winLevelMsg",
   "XBlast.player3.winLevelMsg",
   "XBlast.player4.winLevelMsg",
+  "XBlast.player5.winLevelMsg",
+  "XBlast.player6.winLevelMsg",
 };
 static char *N_PlayerWinLevel[MAX_PLAYER+3] = {
   "xblast.player1.winLevelMsg",
   "xblast.player2.winLevelMsg",
   "xblast.player3.winLevelMsg",
   "xblast.player4.winLevelMsg",
+  "xblast.player5.winLevelMsg",
+  "xblast.player6.winLevelMsg",
   "xblast.singlePlayer.winLevelMsg",
   "xblast.rightPlayer.winLevelMsg",
   "xblast.leftPlayer.winLevelMsg",
@@ -267,6 +299,8 @@ static char *N_PlayerWinGame[MAX_PLAYER+3] = {
   "xblast.player2.winGameMsg",
   "xblast.player3.winGameMsg",
   "xblast.player4.winGameMsg",
+  "xblast.player5.winGameMsg",
+  "xblast.player6.winGameMsg",
   "xblast.singlePlayer.winGameMsg",
   "xblast.rightPlayer.winGameMsg",
   "xblast.leftPlayer.winGameMsg",
@@ -278,6 +312,8 @@ static char *N_PlayerLoseLevel[MAX_PLAYER+3] = {
   "xblast.player2.loseLevelMsg",
   "xblast.player3.loseLevelMsg",
   "xblast.player4.loseLevelMsg",
+  "xblast.player5.loseLevelMsg",
+  "xblast.player6.loseLevelMsg",
   "xblast.singlePlayer.loseLevelMsg",
   "xblast.rightPlayer.loseLevelMsg",
   "xblast.leftPlayer.loseLevelMsg",
@@ -289,6 +325,8 @@ static char *N_PlayerLoseLife[MAX_PLAYER+3] = {
   "xblast.player2.loseLifeMsg",
   "xblast.player3.loseLifeMsg",
   "xblast.player4.loseLifeMsg",
+  "xblast.player5.loseLifeMsg",
+  "xblast.player6.loseLifeMsg",
   "xblast.singlePlayer.loseLifeMsg",
   "xblast.rightPlayer.loseLifeMsg",
   "xblast.leftPlayer.loseLifeMsg",
@@ -300,6 +338,8 @@ static char *N_PlayerGloat[MAX_PLAYER+3] = {
   "xblast.player2.gloatMsg",
   "xblast.player3.gloatMsg",
   "xblast.player4.gloatMsg",
+  "xblast.player5.gloatMsg",
+  "xblast.player6.gloatMsg",
   "xblast.singlePlayer.gloatMsg",
   "xblast.rightPlayer.gloatMsg",
   "xblast.leftPlayer.gloatMsg",
@@ -311,6 +351,8 @@ static char *N_PlayerWelcome[MAX_PLAYER+3] = {
   "xblast.player2.welcomeMsg",
   "xblast.player3.welcomeMsg",
   "xblast.player4.welcomeMsg",
+  "xblast.player5.welcomeMsg",
+  "xblast.player6.welcomeMsg",
   "xblast.singlePlayer.welcomeMsg",
   "xblast.rightPlayer.welcomeMsg",
   "xblast.leftPlayer.welcomeMsg",
@@ -322,12 +364,16 @@ static char *D_PlayerAbort[MAX_PLAYER] = {
   "XBlast.player2.abortMsg",
   "XBlast.player3.abortMsg",
   "XBlast.player4.abortMsg",
+  "XBlast.player5.abortMsg",
+  "XBlast.player6.abortMsg",
 };
 static char *N_PlayerAbort[MAX_PLAYER+3] = {
   "xblast.player1.abortMsg",
   "xblast.player2.abortMsg",
   "xblast.player3.abortMsg",
   "xblast.player4.abortMsg",
+  "xblast.player5.abortMsg",
+  "xblast.player6.abortMsg",
   "xblast.singlePlayer.abortMsg",
   "xblast.rightPlayer.abortMsg",
   "xblast.leftPlayer.abortMsg",
@@ -339,18 +385,21 @@ static char *D_PlayerAbortCancel[MAX_PLAYER] = {
   "XBlast.player2.abortCancelMsg",
   "XBlast.player3.abortCancelMsg",
   "XBlast.player4.abortCancelMsg",
+  "XBlast.player5.abortCancelMsg",
+  "XBlast.player6.abortCancelMsg",
 };
 static char *N_PlayerAbortCancel[MAX_PLAYER+3] = {
   "xblast.player1.abortCancelMsg",
   "xblast.player2.abortCancelMsg",
   "xblast.player3.abortCancelMsg",
   "xblast.player4.abortCancelMsg",
+  "xblast.player5.abortCancelMsg",
+  "xblast.player6.abortCancelMsg",
   "xblast.singlePlayer.abortCancelMsg",
   "xblast.rightPlayer.abortCancelMsg",
   "xblast.leftPlayer.abortCancelMsg",
 };
 #define C_PlayerAbortCancel "XBlast.Player.AbortCancelMsg"
-
 /* player 1 color */
 #define D_PlayerHelmetColor1    "XBlast.player1.helmetColor"
 #define D_PlayerFaceColor1      "XBlast.player1.faceColor"
@@ -379,12 +428,28 @@ static char *N_PlayerAbortCancel[MAX_PLAYER+3] = {
 #define D_PlayerHandsFeetColor4 "XBlast.player4.handsFeetColor"
 #define D_PlayerArmsLegsColor4  "XBlast.player4.armsLegsColor"
 #define D_PlayerBackpackColor4  "XBlast.player4.backpackColor"
+/* player 5 color */
+#define D_PlayerHelmetColor5    "XBlast.player5.helmetColor"
+#define D_PlayerFaceColor5      "XBlast.player5.faceColor"
+#define D_PlayerBodyColor5      "XBlast.player5.bodyColor"
+#define D_PlayerHandsFeetColor5 "XBlast.player5.handsFeetColor"
+#define D_PlayerArmsLegsColor5  "XBlast.player5.armsLegsColor"
+#define D_PlayerBackpackColor5  "XBlast.player5.backpackColor"
+/* player 6 color */
+#define D_PlayerHelmetColor6    "XBlast.player6.helmetColor"
+#define D_PlayerFaceColor6      "XBlast.player6.faceColor"
+#define D_PlayerBodyColor6      "XBlast.player6.bodyColor"
+#define D_PlayerHandsFeetColor6 "XBlast.player6.handsFeetColor"
+#define D_PlayerArmsLegsColor6  "XBlast.player6.armsLegsColor"
+#define D_PlayerBackpackColor6  "XBlast.player6.backpackColor"
 /* all player color names */ 
 static char *N_PlayerHelmetColor[MAX_PLAYER+3] = {
   "xblast.player1.helmetColor",
   "xblast.player2.helmetColor",
   "xblast.player3.helmetColor",
   "xblast.player4.helmetColor",
+  "xblast.player5.helmetColor",
+  "xblast.player6.helmetColor",
   "xblast.singlePlayer.helmetColor",
   "xblast.rightPlayer.helmetColor",
   "xblast.leftPlayer.helmetColor",
@@ -394,6 +459,8 @@ static char *N_PlayerFaceColor[MAX_PLAYER+3] = {
   "xblast.player2.faceColor",
   "xblast.player3.faceColor",
   "xblast.player4.faceColor",
+  "xblast.player5.faceColor",
+  "xblast.player6.faceColor",
   "xblast.singlePlayer.faceColor",
   "xblast.rightPlayer.faceColor",
   "xblast.leftPlayer.faceColor",
@@ -403,6 +470,8 @@ static char *N_PlayerBodyColor[MAX_PLAYER+3] = {
   "xblast.player2.bodyColor",
   "xblast.player3.bodyColor",
   "xblast.player4.bodyColor",
+  "xblast.player5.bodyColor",
+  "xblast.player6.bodyColor",
   "xblast.singlePlayer.bodyColor",
   "xblast.rightPlayer.bodyColor",
   "xblast.leftPlayer.bodyColor",
@@ -412,6 +481,8 @@ static char *N_PlayerHandsFeetColor[MAX_PLAYER+3] = {
   "xblast.player2.handsFeetColor",
   "xblast.player3.handsFeetColor",
   "xblast.player4.handsFeetColor",
+  "xblast.player5.handsFeetColor",
+  "xblast.player6.handsFeetColor",
   "xblast.singlePlayer.handsFeetColor",
   "xblast.rightPlayer.handsFeetColor",
   "xblast.leftPlayer.handsFeetColor",
@@ -421,6 +492,8 @@ static char *N_PlayerArmsLegsColor[MAX_PLAYER+3] = {
   "xblast.player2.armsLegsColor",
   "xblast.player3.armsLegsColor",
   "xblast.player4.armsLegsColor",
+  "xblast.player5.armsLegsColor",
+  "xblast.player6.armsLegsColor",
   "xblast.singlePlayer.armsLegsColor",
   "xblast.rightPlayer.armsLegsColor",
   "xblast.leftPlayer.armsLegsColor",
@@ -430,14 +503,179 @@ static char *N_PlayerBackpackColor[MAX_PLAYER+3] = {
   "xblast.player2.backpackColor",
   "xblast.player3.backpackColor",
   "xblast.player4.backpackColor",
+  "xblast.player5.backpackColor",
+  "xblast.player6.backpackColor",
   "xblast.singlePlayer.backpackColor",
   "xblast.rightPlayer.backpackColor",
   "xblast.leftPlayer.backpackColor",
 };
+
 /* player color class */
 #define C_PlayerColor  "XBlast.Player.Background"
 #endif
 
+/* player attributes array */
+static char **N_PlayerAttributes[] = {
+  /* name */
+  N_Player,
+  /* messages */
+  N_PlayerWinLevel,
+  N_PlayerWinGame,
+  N_PlayerLoseLevel,
+  N_PlayerLoseLife,
+  N_PlayerGloat,
+  N_PlayerWelcome,
+  N_PlayerAbort,
+  N_PlayerAbortCancel,
+  /* colors */
+  N_PlayerHelmetColor,
+  N_PlayerFaceColor,
+  N_PlayerBodyColor,
+  N_PlayerHandsFeetColor,
+  N_PlayerArmsLegsColor,
+  N_PlayerBackpackColor,
+  /* terminator */
+  NULL,
+};
+
+/* player attributes array */
+static char *C_PlayerAttributes[] = {
+  /* name */
+  C_Player,
+  /* messages */
+  C_PlayerWinLevel,
+  C_PlayerWinGame,
+  C_PlayerLoseLevel,
+  C_PlayerLoseLife,
+  C_PlayerGloat,
+  C_PlayerWelcome,
+  C_PlayerAbort,
+  C_PlayerAbortCancel,
+  /* colors */
+  C_PlayerColor,
+  C_PlayerColor,
+  C_PlayerColor,
+  C_PlayerColor,
+  C_PlayerColor,
+  C_PlayerColor,
+  /* terminator */
+  NULL,
+};
+
+/* player key resources */
+
+/* single player */
+#define D_SinglePlayerPauseKey       "XBlast.singlePlayer.pauseKey"
+#define D_SinglePlayerUpKey          "XBlast.singlePlayer.upKey"
+#define D_SinglePlayerLeftKey        "XBlast.singlePlayer.leftKey"
+#define D_SinglePlayerDownKey        "XBlast.singlePlayer.downKey"
+#define D_SinglePlayerRightKey       "XBlast.singlePlayer.rightKey"
+#define D_SinglePlayerStopKey        "XBlast.singlePlayer.stopKey"
+#define D_SinglePlayerBombKey        "XBlast.singlePlayer.bombKey"
+#define D_SinglePlayerSpecialKey     "XBlast.singlePlayer.specialKey"
+#define D_SinglePlayerAbortKey       "XBlast.singlePlayer.abortKey"
+#define D_SinglePlayerAbortCancelKey "XBlast.singlePlayer.abortCancelKey"
+/* right player */
+#define D_RightPlayerPauseKey       "XBlast.rightPlayer.pauseKey"
+#define D_RightPlayerUpKey          "XBlast.rightPlayer.upKey"
+#define D_RightPlayerLeftKey        "XBlast.rightPlayer.leftKey"
+#define D_RightPlayerDownKey        "XBlast.rightPlayer.downKey"
+#define D_RightPlayerRightKey       "XBlast.rightPlayer.rightKey"
+#define D_RightPlayerStopKey        "XBlast.rightPlayer.stopKey"
+#define D_RightPlayerBombKey        "XBlast.rightPlayer.bombKey"
+#define D_RightPlayerSpecialKey     "XBlast.rightPlayer.specialKey"
+#define D_RightPlayerAbortKey       "XBlast.rightPlayer.abortKey"
+#define D_RightPlayerAbortCancelKey "XBlast.rightPlayer.abortCancelKey"
+/* left player */
+#define D_LeftPlayerPauseKey       "XBlast.leftPlayer.pauseKey"
+#define D_LeftPlayerUpKey          "XBlast.leftPlayer.upKey"
+#define D_LeftPlayerLeftKey        "XBlast.leftPlayer.leftKey"
+#define D_LeftPlayerDownKey        "XBlast.leftPlayer.downKey"
+#define D_LeftPlayerRightKey       "XBlast.leftPlayer.rightKey"
+#define D_LeftPlayerStopKey        "XBlast.leftPlayer.stopKey"
+#define D_LeftPlayerBombKey        "XBlast.leftPlayer.bombKey"
+#define D_LeftPlayerSpecialKey     "XBlast.leftPlayer.specialKey"
+#define D_LeftPlayerAbortKey       "XBlast.leftPlayer.abortKey"
+#define D_LeftPlayerAbortCancelKey "XBlast.leftPlayer.abortCancelKey"
+
+/* all player key names */
+static char* N_PlayerPauseKey[3] = {
+  "xblast.singlePlayer.pauseKey",
+  "xblast.rightPlayer.pauseKey",
+  "xblast.leftPlayer.pauseKey",
+};
+static char* N_PlayerUpKey[3] = {
+  "xblast.singlePlayer.upKey",
+  "xblast.rightPlayer.upKey",
+  "xblast.leftPlayer.upKey",
+};
+static char* N_PlayerLeftKey[3] = {
+  "xblast.singlePlayer.leftKey",
+  "xblast.rightPlayer.leftKey",
+  "xblast.leftPlayer.leftKey",
+};
+static char* N_PlayerRightKey[3] = {
+  "xblast.singlePlayer.rightKey",
+  "xblast.rightPlayer.rightKey",
+  "xblast.leftPlayer.rightKey",
+};
+static char* N_PlayerDownKey[3] = {
+  "xblast.singlePlayer.downKey",
+  "xblast.rightPlayer.downKey",
+  "xblast.leftPlayer.downKey",
+};
+static char* N_PlayerStopKey[3] = {
+  "xblast.singlePlayer.stopKey",
+  "xblast.rightPlayer.stopKey",
+  "xblast.leftPlayer.stopKey",
+};
+static char* N_PlayerBombKey[3] = {
+  "xblast.singlePlayer.bombKey",
+  "xblast.rightPlayer.bombKey",
+  "xblast.leftPlayer.bombKey",
+};
+static char* N_PlayerSpecialKey[3] = {
+  "xblast.singlePlayer.specialKey",
+  "xblast.rightPlayer.specialKey",
+  "xblast.leftPlayer.specialKey",
+};
+static char* N_PlayerAbortKey[3] = {
+  "xblast.singlePlayer.abortKey",
+  "xblast.rightPlayer.abortKey",
+  "xblast.leftPlayer.abortKey",
+};
+static char* N_PlayerAbortCancelKey[3] = {
+  "xblast.singlePlayer.abortCancelKey",
+  "xblast.rightPlayer.abortCancelKey",
+  "xblast.leftPlayer.abortCancelKey",
+};
+
+/* classes for keys */
+#define C_PlayerPauseKey       "XBlast.Player.PauseKey"
+#define C_PlayerUpKey          "XBlast.Player.UpKey"
+#define C_PlayerLeftKey        "XBlast.Player.LeftKey"
+#define C_PlayerRightKey       "XBlast.Player.RightKey"
+#define C_PlayerDownKey        "XBlast.Player.DownKey"
+#define C_PlayerStopKey        "XBlast.Player.StopKey"
+#define C_PlayerBombKey        "XBlast.Player.BombKey"
+#define C_PlayerSpecialKey     "XBlast.Player.SpecialKey"
+#define C_PlayerAbortKey       "XBlast.Player.AbortKey"
+#define C_PlayerAbortCancelKey "XBlast.Player.AbortCancelKey"
+
+#ifdef XBLAST_SOUND
+/*
+ * sound server resources
+ */
+#define W_SoundServer ".soundServer"
+#define D_SoundServer "XBlast.soundServer"
+#define N_SoundServer "xblast.soundServer"
+#define C_SoundServer "XBlast.SoundServer"
+#endif
+
 /*
  * end of file
  */
+
+
+
+
