@@ -1,10 +1,10 @@
 /*
- * Programm XBLAST V1.2.14 or higher
+ * Programm XBLAST V2.2 or higher
  * (C) by Oliver Vogel (e-mail: vogel@ikp.uni-koeln.de)
- * May 7th 1996
+ * December 9th 1996
  * started August 1993
  *
- * File: xsetup.c 
+ * File: setup.c 
  * setup for xblast
  *
  *
@@ -45,28 +45,39 @@
 #include "maze.h"
 #include "resource.h"
 #include "defaults.h"
+#include "pipe.h"
+#include "data.h"
+#include "main.h"
 
 /*
  * extern varaiables
  */ 
-extern Display *dpy[MAX_PLAYER];
+extern Display *dpy[MAX_DISPLAY];
 
 /*
  * local varaiables
  */
-
 static char *prog_name = NULL;
 
+/*
+ * local function pointers
+ */
+#ifdef __STDC__
+static XrmDatabase (*create_player_database)(XBConfig *config);
+#else
+static XrmDatabase (*create_player_database)();
+#endif
+
 /* Databases */
 static XrmDatabase default_DB = NULL;
 static XrmDatabase arg_DB = NULL;
 
 static XrmDatabase disp_DB[MAX_PLAYER] = { 
-  NULL, NULL, NULL, NULL 
+  NULL, NULL, NULL, NULL, NULL
 };
 
 /*
- * local function dup_string
+ * public function dup_string
  * (not all systems have strdup)
  */
 #ifdef __STDC__
@@ -146,6 +157,49 @@ equal_string (s1, s2)
 }
 
 
+/*
+ * local function numeric_string
+ */
+#ifdef __STDC__
+static int
+string_to_interval (char *string, 
+		    int *left, 
+		    int *right) 
+#else
+static int
+string_to_interval (string, left, right)
+     char *string;
+     int *left, *right;
+#endif
+{
+  char *ptr;
+  int flag = FALSE;
+
+  for (ptr = string; *ptr != '\0'; ptr++) {
+    if (!isdigit(*ptr)) {
+      if (flag || ('-' != *ptr) ) {
+	return -1;
+      } 
+      flag = TRUE;
+    }
+  }
+
+  if (flag) {
+    /* string is an interval */
+    sscanf(string, "%d-%d", left,right);
+  } else {
+    /* string is single number */
+    *left = *right = atoi (string);
+  }
+
+#if DEBUG
+  fprintf(stderr, "Scanned interval [%d,%d]\n",*left, *right);
+#endif
+  return 0;
+}
+
+
+#if 0
 /*
  * local function numeric_string
  */
@@ -165,6 +219,79 @@ numeric_string (ptr)
   }
   return 1;
 }
+#endif
+
+/*
+ * public function split_string
+ */
+#ifdef __STDC__
+char ** 
+split_string (char *string,
+	      int *largc)
+#else
+char ** 
+split_string (string, largc)
+     char *string;
+     int *largc;
+#endif
+{
+  void *ptr;
+  char *buf;
+  char **argv;
+  int i, flag, length, size, argc;
+
+  /* get number of words */
+  flag = TRUE;
+  *largc = 0;
+  size = 0;
+  length = strlen(string);
+  for (i=0; i<length; i++) {
+    if (isspace(string[i])) {
+      flag = TRUE;
+    } else {
+      if (flag) {
+	(*largc) ++;
+	flag = FALSE;
+      }
+      size ++;
+    }
+  }
+
+  /* alloc array */
+  if (NULL == (ptr = malloc( ((*largc)+1) * sizeof(char *) 
+			    + (size+(*largc)) * sizeof(char) )) ) {
+    fprintf(stderr, "string alloc failed\n");
+    exit_prg(1);
+  }
+  
+  /* now store strings */
+  buf  = (char *) ((char **)ptr + ((*largc)+1));
+  argv = (char **) ptr;
+  argc = 0;
+  flag=TRUE;
+  for (i=0; i<length; i++) {
+    if (isspace(string[i])) {
+      if (!flag) {
+	flag = TRUE;
+	*buf = '\0';
+	buf++;
+      }
+    } else {
+      if (flag) {
+	argv[argc]=buf;
+	argc++;
+	flag = FALSE;
+      }
+      *buf = string[i];
+      buf++;
+    }
+  }
+  *buf='\0';
+  argv[argc]=NULL;
+  
+
+  return argv;
+}
 
 
 
@@ -444,36 +571,65 @@ query_string_for_database (db, res_name, query_string, def)
   int length;
   char *ptr, *last;
 
-#if 0
-  while (1) {
-#endif
-    if (def == NULL) {
-      printf("%s : ", query_string);
-    } else {
-      printf("%s (default is \"%s\"): ", query_string, def);
-    }
-    fgets(zeile, 1024, stdin);
-    /* eliminate first white spaces */
-    length = strlen(zeile);
-    last = zeile + length - 1;
-    if (*last == '\n') {
-      *last = '\0';
-    }
-
-    for (ptr = zeile; ptr != last && isspace(*ptr); ptr ++);
-    if (ptr != last) {
-      /* use input for database */
-      XrmPutStringResource(db, res_name, ptr);
-      return dup_string(ptr, strlen(ptr));
-    } else if (def != NULL) {
-      XrmPutStringResource(db, res_name, def);
-      return def;
-    } else {
-      return NULL;
-    }
-#if 0
+  if (def == NULL) {
+    printf("%s : ", query_string);
+  } else {
+    printf("%s (default is \"%s\"): ", query_string, def);
   }
+  fgets(zeile, 1024, stdin);
+  /* eliminate first white spaces */
+  length = strlen(zeile);
+  last = zeile + length - 1;
+  if (*last == '\n') {
+    *last = '\0';
+  }
+  
+  for (ptr = zeile; ptr != last && isspace(*ptr); ptr ++);
+  if (ptr != last) {
+    /* use input for database */
+    XrmPutStringResource(db, res_name, ptr);
+    return dup_string(ptr, strlen(ptr));
+  } else if (def != NULL) {
+    XrmPutStringResource(db, res_name, def);
+    return def;
+  } else {
+    return NULL;
+  }
+}
+
+
+/*
+ * local function query_list_for_database
+ */
+#ifdef __STDC__
+static char *
+query_list_for_database (XrmDatabase *db,
+			 char *resname,
+			 char *query_string,
+			 char **str_list)
+#else
+static char *
+query_list_for_database (db, resname, query_string, str_list)
+     XrmDatabase *db;
+     char *resname;
+     char *query_string;
+     char **str_list;
 #endif
+{
+  int nlist;
+  char *result;
+
+  printf("%s:\n", query_string);
+  for (nlist = 0; str_list[nlist] != NULL; nlist ++) {
+    printf(" [%d]\t%s\n", nlist+1, str_list[nlist]);
+  }
+
+  result = str_list[query_int_for_database
+		    (db, "xblast.ignore", "Your choice", 1, nlist, 1) - 1];
+
+  XrmPutStringResource(db, resname, result);
+  
+  return result;
 }
 
 
@@ -493,8 +649,8 @@ print_usage (prog, option)
 {
   fprintf (stderr, "%s: unknown option \"%s\"\n", prog, option);
   fprintf (stderr, "Usage: %s [player[@display] ...] [options]\n", prog);
-  fprintf (stderr, "Type \"%s -?\" for a list of options\n", prog);
-  exit(1);
+  fprintf (stderr, "Type \"man xblast\" for a list of options\n");
+  exit_prg(1);
 }
 
 
@@ -504,18 +660,18 @@ print_usage (prog, option)
  */
 #ifdef __STDC__
 static XrmDatabase 
-set_defaults (void)
+set_defaults (BMEntry *ptr)
 #else
 static XrmDatabase 
-set_defaults ()
+set_defaults (ptr)
+     BMEntry *ptr;
 #endif
 {
   XrmDatabase internal_DB = NULL;
   XrmDatabase appl_DB = NULL;
-  BMEntry *ptr;
 
   /* get hardcoded defaults */
-  for (ptr = default_setting; ptr->name != NULL; ptr++) {
+  for (; ptr->name != NULL; ptr++) {
     XrmPutStringResource(&internal_DB, ptr->name, ptr->value);
   }
 
@@ -554,7 +710,7 @@ parse_player_config (argc, argv)
     if (i >MAX_PLAYER) {
       fprintf (stderr, "%s: Too many players defined. The maximum is %d\n",
 	       prog_name, MAX_PLAYER);
-      exit (1);
+      exit_prg(1);
     }
     /* get player and display name */
     p_name = argv[i];
@@ -603,7 +759,8 @@ parse_level_usage (argc, argv)
   XrmDatabase dont_DB = NULL;
   XrmDatabase use_DB = NULL;
   char *flag;
-  int i,j,level;
+  int i,j;
+  int left, right, level;
 
   if (*argc < 2) {
     return NULL;
@@ -614,7 +771,7 @@ parse_level_usage (argc, argv)
     XrmPutStringResource(&use_DB, C_UseLevel, "False");
     flag = "True";
   } else if (0 == strcmp("-U", argv[1]) ) {
-    for (level = 0; level <LEVEL_MAX; level ++) {
+    for (level = 0; level <levelMax; level ++) {
       XrmPutStringResource(&dont_DB, get_level_res_name(level), "False");
     }
     flag = "True";
@@ -622,7 +779,7 @@ parse_level_usage (argc, argv)
     XrmPutStringResource(&use_DB, C_UseLevel, "True");
     flag = "False";
   } else if (0 == strcmp("+U", argv[1]) ) {
-    for (level = 0; level <LEVEL_MAX; level ++) {
+    for (level = 0; level <levelMax; level ++) {
       XrmPutStringResource(&dont_DB, get_level_res_name(level), "True");
     }
     flag = "False";
@@ -630,17 +787,19 @@ parse_level_usage (argc, argv)
     return NULL;
   }
 
-  for (i=2; (i<*argc) && numeric_string(argv[i]); i++) {
-    level = atoi(argv[i]);
-    if ( (level < 0) || (level > LEVEL_MAX) ) {
-      fprintf (stderr, "%s: level %d defined for \"%s\" out of range\n", prog_name, 
-	       level, argv[1]);
+  for (i=2; (i<*argc) && ! string_to_interval(argv[i],&left,&right); i++) {
+    for (level = left; level <=right; level ++) {
+      if ( (level < 0) || (level > levelMax) ) {
+	fprintf (stderr, "%s: level %d defined for \"%s\" out of range\n", 
+		 prog_name, level, argv[1]);
+	continue;
+      }
+      XrmPutStringResource(&use_DB, get_level_res_name(level), flag);
     }
-    XrmPutStringResource(&use_DB, get_level_res_name(level), flag);
   }
 
   if (2 == i) {
-    fprintf (stderr, "%s: Warning! Define at least one level for option \"%s\"\n",
+    fprintf (stderr,"%s: Warning! Define at least one level for option \"%s\"\n",
 	     prog_name, argv[1]);
   }
 
@@ -775,11 +934,11 @@ setup_exists (name)
  */ 
 #ifdef __STDC__
 static XrmDatabase 
-load_setup (XrmDatabase arg_DB)
+load_setup (XrmDatabase DB)
 #else
 static XrmDatabase 
-load_setup (arg_DB)
-     XrmDatabase arg_DB;
+load_setup (DB)
+     XrmDatabase DB;
 #endif
 {
   char *home, *fname;
@@ -790,7 +949,7 @@ load_setup (arg_DB)
   path[0] = '\0';
 
   /* check load mode */
-  if (NULL != (fname = db_get_string(arg_DB, N_LoadNamedDefaults, 
+  if (NULL != (fname = db_get_string(DB, N_LoadNamedDefaults, 
 				     C_LoadNamedDefaults) ) ) {
     /* load named default */
     if (NULL != (home = getenv("HOME") ) ) {
@@ -804,7 +963,7 @@ load_setup (arg_DB)
     }
     strcat(path,"/");
     strcat(path,fname);
-  } else if (db_get_bool(arg_DB, N_LoadDefaults, C_LoadDefaults) ) {
+  } else if (db_get_bool(DB, N_LoadDefaults, C_LoadDefaults) ) {
     if (NULL != (home = getenv("HOME") ) ) {
       strcpy(path, home);
     }
@@ -844,73 +1003,85 @@ save_setup (config, setup, st)
   char *home;
   char *fname;
   FILE *fp;
-  int i;
+  int i, num_player;
   char path[1024];
   static char* int_format = "%s: %d\n";
   static char* str_format = "%s: %s\n";
 
   /* check if setup has to be saved */
-  if (NULL != (fname = db_get_string(default_DB, N_SaveNamedDefaults,
-				     C_SaveNamedDefaults) )) {
-    if (NULL != (home = getenv("HOME") ) ) {
-      strcpy(path, home);
-    }
-    strcat(path, file_setup_dir);
-    if (check_create_dir(path)) {
-      fprintf(stderr, "%s: Warning! Failed to create setup dir\"%s\"\n", 
-	      prog_name, fname);
+  if (CM_Child != config->com_mode) {
+    if (NULL != (fname = db_get_string(default_DB, N_SaveNamedDefaults,
+				       C_SaveNamedDefaults)))  {
+      if (NULL != (home = getenv("HOME") ) ) {
+	strcpy(path, home);
+      }
+      strcat(path, file_setup_dir);
+      if (check_create_dir(path)) {
+	fprintf(stderr, "%s: Warning! Failed to create setup dir\"%s\"\n", 
+		prog_name, fname);
+	return;
+      }
+      strcat(path,"/");
+      strcat(path,fname);
+    } else if (db_get_bool(default_DB, N_SaveDefaults, C_SaveDefaults) )  {
+      /* set file name */
+      if (NULL != (home = getenv("HOME") ) ) {
+	strcpy (path, home);
+      }
+      strcat(path, file_setup);
+    } else {
       return;
     }
-    strcat(path,"/");
-    strcat(path,fname);
-  } else if (db_get_bool(default_DB, N_SaveDefaults, C_SaveDefaults) ) {
-    /* set file name */
-    if (NULL != (home = getenv("HOME") ) ) {
-      strcpy (path, home);
+    
+    /* open file for output */
+    if (NULL == (fp = fopen(path,"w"))) {
+      fprintf (stderr, "%s: warning failed to open setup file \"%s\"\n",
+	       prog_name, path);
+      return;
     }
-    strcat(path, file_setup);
-  } else {
-    return;
-  }
-
-  /* open file for output */
-  if (NULL == (fp = fopen(path,"w"))) {
-    fprintf (stderr, "%s: warning failed to open setup file \"%s\"\n",
-	     prog_name, path);
-    return;
-  }
-
-  /* write config and string resources */
-  fprintf(fp, int_format, N_NumberOfPlayers,  config->num_player);
-  for (i=0; i<config->num_player; i++) {
-    if (NULL != config->display[config->pl_at_disp[i]]) {
-      fprintf(fp, str_format, N_AtDisplay[i],  
-	      config->display[config->pl_at_disp[i]]);
+    
+    /* write config and string resources */
+    if (config->team_mode == TM_Double) { 
+      num_player = config->num_player / 2;
+    } else {
+      num_player = config->num_player;
     }
-    fprintf(fp, str_format, N_Player[i], st[i].name);
+    fprintf(fp, int_format, N_NumberOfPlayers,  num_player);
+    for (i=0; i<num_player; i++) {
+      if (NULL != config->display[config->pl_at_disp[i]]) {
+	fprintf(fp, str_format, N_AtDisplay[i],  
+		config->display[config->pl_at_disp[i]]);
+      }
+      fprintf(fp, str_format, N_Player[i], st[i].name);
+    }
+    switch (config->team_mode) {
+    case TM_Single: fprintf(fp, str_format, N_TeamMode, "None"); break;
+    case TM_Team:   fprintf(fp, str_format, N_TeamMode, "Team"); break;
+    case TM_Double: fprintf(fp, str_format, N_TeamMode, "Double"); break;
+    }
+    
+    /* write game setup */
+    fprintf(fp, int_format, N_NumberOfVictories, setup->max_victories);
+    fprintf(fp, int_format, N_NumberOfLives, setup->max_lives);
+    fprintf(fp, int_format, N_StartingLevel, setup->start_level);
+    fprintf(fp, str_format, N_RandomLevelOrder, 
+	    setup->random_mode ? "True" : "False" );
+    fprintf(fp, str_format, N_RandomPlayerPosition, 
+	    setup->random_spos ? "True" : "False" );
+    fprintf(fp, str_format, N_BellSound, setup->sound_flag ? "True" : "False" );
+    fprintf(fp, int_format, N_FrameRate, (int)(1000000/setup->frame_time));
+    fprintf(fp, str_format, N_AllowColorMode, 
+	    setup->color_mode ? "True" : "False" );
+    
+    /* write level usage */
+    for (i=0; i<levelMax; i++)  {
+      fprintf(fp, str_format, get_level_res_name(i), 
+	      setup->use_level[i] ? "True" : "False");
+    }
+    
+    /* close file */
+    fclose(fp);
   }
-
-  /* write game setup */
-  fprintf(fp, int_format, N_NumberOfVictories, setup->max_victories);
-  fprintf(fp, int_format, N_NumberOfLives, setup->max_lives);
-  fprintf(fp, int_format, N_StartingLevel, setup->start_level);
-  fprintf(fp, str_format, N_RandomLevelOrder, 
-	  setup->random_mode ? "True" : "False" );
-  fprintf(fp, str_format, N_RandomPlayerPosition, 
-	  setup->random_spos ? "True" : "False" );
-  fprintf(fp, str_format, N_BellSound, setup->sound_flag ? "True" : "False" );
-  fprintf(fp, int_format, N_FrameRate, (int)(1000000/setup->frame_time));
-  fprintf(fp, str_format, N_AllowColorMode, 
-	  setup->color_mode ? "True" : "False" );
-
-  /* write level usage */
-  for (i=0; i<LEVEL_MAX; i++)  {
-    fprintf(fp, str_format, get_level_res_name(i), 
-	    setup->use_level[i] ? "True" : "False");
-  }
-
-  /* close file */
-  fclose(fp);
 }
 
 
@@ -966,7 +1137,8 @@ parse_commandline (argc, argv)
   prog_name = argv[0];
 
   /* initiate default values */
-  default_DB = set_defaults();
+  default_DB = set_defaults(default_settings);
+  arg_DB = set_defaults(arg_settings);
 
   /* parse command line for player config */
 #ifdef DEBUG
@@ -993,10 +1165,6 @@ parse_commandline (argc, argv)
   }
 
   /* check for terminating options */
-  if (db_get_bool(arg_DB, N_PrintLevels, C_PrintLevels )) {
-    show_levels();
-    exit(0);
-  }
   if (db_get_bool(arg_DB, N_PrintHelp, C_PrintHelp )) {
     print_usage (argv[0], "-?");
   }
@@ -1005,6 +1173,16 @@ parse_commandline (argc, argv)
   setup_DB = load_setup(arg_DB);
   XrmCombineDatabase(setup_DB, &arg_DB, FALSE);
 
+  /* show list of levels now */
+  if (db_get_bool(arg_DB, N_PrintLevels, C_PrintLevels )) {
+    show_levels();
+    exit_prg(0);
+  }
+  if (db_get_bool(arg_DB, N_PrintLevelsTcl, C_PrintLevelsTcl )) {
+    show_levels_tcl();
+    exit_prg(0);
+  }
+
   /* merge databases */
   XrmMergeDatabases(player_DB, &arg_DB);
 }
@@ -1049,8 +1227,8 @@ interactive_setup (num_player)
   if (!query_bool_for_database (&inter_DB, N_RandomLevelOrder, 
 				"Random level order")) {
     query_int_for_database (&inter_DB, N_StartingLevel, "Starting Level", 
-			    0, LEVEL_MAX-1, 
-			    db_get_int(default_DB, 0, LEVEL_MAX-1, 
+			    0, levelMax-1, 
+			    db_get_int(default_DB, 0, levelMax-1, 
 				       N_StartingLevel, C_StartingLevel) );
   }
   
@@ -1059,7 +1237,7 @@ interactive_setup (num_player)
     int level;
 
     printf("Select levels to play:\n");
-    for (level = 0; level <LEVEL_MAX; level++) {
+    for (level = 0; level <levelMax; level++) {
       query_bool_for_database(&inter_DB, get_level_res_name(level),
 			      get_level_name(level));
     }
@@ -1087,6 +1265,13 @@ interactive_setup (num_player)
  *   part I one the interactive setup, just queries number of players 
  *   and display names.
  */
+static char *team_mode_list[] = {
+  "None",
+  "Team",
+  "Double",
+  NULL,
+};
+
 #ifdef __STDC__
 int 
 interactive_config (char *argv0) 
@@ -1105,28 +1290,24 @@ interactive_config (argv0)
   prog_name = argv0;
 
   /* initiate default values */
-  default_DB = set_defaults();
+  default_DB = set_defaults(default_settings);
+  arg_DB = set_defaults(arg_settings);
 
   /* load defaults from file and return */
   if (query_bool_for_database(&arg_DB, "xblast.ignore", "Load an old setup")) {
-#if 0
-    if (!query_bool_for_database(&arg_DB, N_LoadDefaults, 
-				 "Load the default setup")) {
-#endif
-      /* load a named setup */
-      if (NULL != (def_setup = list_setups() ) ) {
-	while ( ! setup_exists (query_string_for_database
-				(&arg_DB, N_LoadNamedDefaults,
-				 "Name of setup", def_setup) ) ); 
-      } else {
-	printf("Sorry, no named setup available.\n");
-      }
-#if 0
+    /* load a named setup */
+    if (NULL != (def_setup = list_setups() ) ) {
+      while ( ! setup_exists (query_string_for_database
+			      (&arg_DB, N_LoadNamedDefaults,
+			       "Name of setup", def_setup) ) ); 
+    } else {
+      printf("Sorry, no named setup available.\n");
     }
-#endif
     db_get_string(arg_DB, N_LoadNamedDefaults, C_LoadNamedDefaults);
     arg_DB = load_setup(arg_DB);
-    return 0;
+    
+
+    return 1;
   }
 
   num_player = query_int_for_database 
@@ -1138,6 +1319,14 @@ interactive_config (argv0)
     query_string_for_database (&arg_DB, N_AtDisplay[player], zeile, NULL);
   }
 
+  /* get team mode */
+  query_list_for_database (&arg_DB, N_TeamMode, "Select team mode", 
+			   team_mode_list);
+
+  /* ask for fork mode */
+  query_bool_for_database(&arg_DB, N_Fork, "Use fork mode");
+
+
   return 1;
 }
 
@@ -1157,7 +1346,7 @@ config_from_database (config)
 {
   int disp, player;
   char *d_name;
-  
+  char *m_name;
   
   /* get number of players */
   config->num_player 
@@ -1174,12 +1363,43 @@ config_from_database (config)
       config->num_disp ++;
     }
   }
-  /*  no old style x resources */
-  config->use_X_resources = FALSE;
+
   /* set default displa (not finished) */
   config->default_disp = 0;
+
+  /* fork mode */
+  config->fork = db_get_bool(arg_DB, N_Fork, C_Fork);
+  
+  /* team mode */
+  m_name = db_get_string(arg_DB, N_TeamMode, C_TeamMode);
+  if (equal_string(m_name, "Team")) {
+    config->team_mode = TM_Team;
+  } else if (equal_string(m_name, "Double")) {
+    config->team_mode = TM_Double;
+  } else if (equal_string(m_name, "None")) {
+    config->team_mode = TM_Single;
+  } else {
+    fprintf(stderr, "Unknown team mode \"%s\".\n", m_name);
+    exit_prg(1);
+  }
 }
 
+#ifdef XBLAST_SOUND
+/*
+ * global function sound_server_from_database
+ */
+#ifdef __STDC__
+char *
+sound_server_from_database (void)
+#else
+char *
+sound_server_from_database ()
+#endif
+{
+  return db_get_string(arg_DB, N_SoundServer, C_SoundServer);
+}
+#endif
+
 
 
 /*
@@ -1206,26 +1426,248 @@ copy_database_entry (src_DB, dst_DB, name, class, dst_name)
   
   /* try to get entry */
   if (NULL != (result = db_get_string(src_DB, name, class) ) ) {
-    /* put into destination datbase */
+    /* put into destination database */
     XrmPutStringResource(dst_DB, dst_name, result);
   }
 }
 
 
+
+/*
+ * local function: get_database_entry_from_child
+ */ 
+#ifdef __STDC__
+static void 
+get_database_entry_from_child (XrmDatabase *db, 
+			       char *attr)
+#else
+static void 
+get_database_entry_from_child (db, attr)
+     XrmDatabase *db;
+     char *attr;
+#endif
+{
+  char buffer[1024];
+
+  /* get string from child */
+  string_from_children (buffer);
+#ifdef DEBUG_IPC
+  fprintf(stderr, "C->P: %s stored in %s\n", buffer, attr);
+#endif
+  if (0 != strlen(buffer)) {
+    /* store string in database */
+    XrmPutStringResource(db, attr, buffer);
+  }
+}
+
+
+
+/*
+ * local function: send_database_entry_to_children
+ */ 
+#ifdef __STDC__
+static void 
+send_database_entry_to_children (XrmDatabase db, 
+				 char *name,
+				 char *class)
+#else
+static void 
+send_database_entry_to_children (db, name, class)
+     XrmDatabase db;
+     char *name, *class;
+#endif
+{
+  char *buffer;
+  char dummy = '\0';
+
+  if (NULL != (buffer = db_get_string (db, name, class))) {
+    string_to_children (buffer);
+  } else {
+    string_to_children (&dummy);
+  }
+#ifdef DEBUG_IPC
+  fprintf(stderr, "P->C: %s from %s\n", buffer, name);
+#endif
+}
+
+
+
+/*
+ * local function: parent_create_player_DB
+ */
+#ifdef __STDC
+static XrmDatabase
+parent_create_player_DB (XBConfig *config)
+#else
+static XrmDatabase
+parent_create_player_DB (config)
+     XBConfig *config;
+#endif
+{
+  XrmDatabase player_DB = NULL;
+  int i, player;
+  int src, p1, p2;
+
+  p1 = config->disp_player[0].p1;
+  p2 = config->disp_player[0].p2;
+  src = (p1==p2) ? SINGLE_PLAYER : RIGHT_PLAYER;
+#ifdef DEBUG_IPC
+  fprintf(stderr, "Parent CDB %d %d\n", p1, p2);
+#endif
+
+  for (player=0; player < config->num_player; player ++) {
+#ifdef DEBUG_IPC
+  fprintf(stderr, "Parent CDB [%d]\n", player);
+#endif
+    
+    /* retrievs all attributes for one player */
+    if (p1 == player) {
+      /* get local data for player */
+      for (i=0; N_PlayerAttributes[i] != NULL; i++) {
+	copy_database_entry(disp_DB[0], &player_DB, 
+			    N_PlayerAttributes[i][src], 
+			    C_PlayerAttributes[i], 
+			    N_PlayerAttributes[i][player]);
+      }
+      p1 = p2;
+      src = LEFT_PLAYER;
+    } else {
+      /* get remote data for player */
+      for (i=0; N_PlayerAttributes[i] != NULL; i++) {
+	get_database_entry_from_child (&player_DB,N_PlayerAttributes[i][player]);
+      }
+    }
+    /* send to all children */
+    for (i=0; N_PlayerAttributes[i] != NULL; i++) {
+      send_database_entry_to_children (player_DB, 
+				       N_PlayerAttributes[i][player],
+				       C_PlayerAttributes[i]);
+    }
+  }
+  return player_DB;
+}
+
+
+
+/*
+ * local function: send_database_entry_to_parent
+ */
+#ifdef __STDC__
+static void
+send_database_entry_to_parent (XrmDatabase db,
+			       char *name,
+			       char *class)
+#else
+static void
+send_database_entry_to_parent (db, name, class)
+     XrmDatabase db;
+     char *name, *class;
+#endif
+{
+  char *buffer;
+  char dummy = '\0';
+
+  if (NULL != (buffer = db_get_string(db, name, class))) {
+    string_to_parent (buffer);
+  } else {
+    /* send empty string */
+    string_to_parent (&dummy);
+  }
+#ifdef DEBUG_IPC
+  fprintf(stderr, "C->P: %s from %s\n", buffer, name);
+#endif
+}
+
+
+
+#ifdef __STDC__
+static void 
+get_database_entry_from_parent (XrmDatabase *db, 
+				char *attr)
+#else
+static void 
+get_database_entry_from_parent (db, attr)
+     XrmDatabase *db;
+     char *attr;
+#endif
+{
+  char buffer[1024];
+  
+  /* get string from parent */
+  string_from_parent(buffer);
+#ifdef DEBUG_IPC
+  fprintf(stderr, "P->C: %s stored in %s\n", buffer, attr);
+#endif
+  if (0 != strlen(buffer)) {
+    /* store string in database */
+    XrmPutStringResource(db, attr, buffer);
+  }
+}
+
+
+
+/*
+ * local function: child_create_player_DB
+ */
+#ifdef __STDC
+static XrmDatabase
+child_create_player_DB (XBConfig *config)
+#else
+static XrmDatabase
+child_create_player_DB (config)
+     XBConfig *config;
+#endif
+{
+  XrmDatabase player_DB = NULL;
+  int i, player, p1, p2, src;
+
+  /* get local players */
+  p1 = config->disp_player[0].p1;
+  p2 = config->disp_player[0].p2;
+  src = (p1 == p2) ? SINGLE_PLAYER : RIGHT_PLAYER;
+
+#ifdef DEBUG_IPC
+  fprintf(stderr, "Child CDB %d %d\n", p1, p2);
+#endif
+
+  for (player=0; player < config->num_player; player ++) {
+#ifdef DEBUG_IPC
+  fprintf(stderr, "Child CDB [%d]\n", player);
+#endif
+
+    if (player == p1) {
+      for (i=0; N_PlayerAttributes[i]!=NULL; i++) {
+	send_database_entry_to_parent (disp_DB[0], 
+				       N_PlayerAttributes[i][src],
+				       C_PlayerAttributes[i]);
+      }
+      /* set index for possible next player */
+      p1 = p2;
+      src = LEFT_PLAYER;
+    }
+    for (i=0; N_PlayerAttributes[i]!=NULL; i++) {
+      get_database_entry_from_parent (&player_DB, N_PlayerAttributes[i][player]);
+    }
+  }
+  return player_DB;
+}
+
+
+
 /*
  * local function: create_player_database
  */
 #ifdef __STDC
 static XrmDatabase
-create_player_database (XBConfig *config)
+local_create_player_DB (XBConfig *config)
 #else
 static XrmDatabase
-create_player_database (config)
+local_create_player_DB (config)
      XBConfig *config;
 #endif
 {
   XrmDatabase player_DB = NULL;
-  int disp, player, src, dst;
+  int i, disp, player, src, dst;
 
   for (disp=0; disp < config->num_disp; disp++) {
     for (player=0; player < config->disp_player[disp].num; player ++) {
@@ -1246,53 +1688,13 @@ create_player_database (config)
       fprintf(stderr, "Copy resources: Player(%d)@Display(%d) to [player%d]\n",
 	      src, disp, dst);
 #endif      
-      /* copy data, first of all the name */
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			 N_Player[src], C_Player, N_Player[dst]);
-      /* now the player strings */
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerWinLevel[src], C_PlayerWinLevel, 
-			  N_PlayerWinLevel[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			 N_PlayerWinGame[src], C_PlayerWinGame, 
-			  N_PlayerWinGame[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerLoseLevel[src], C_PlayerLoseLevel, 
-			  N_PlayerLoseLevel[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerLoseLife[src], C_PlayerLoseLevel, 
-			  N_PlayerLoseLife[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerGloat[src], C_PlayerGloat, 
-			  N_PlayerGloat[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerWelcome[src], C_PlayerWelcome, 
-			  N_PlayerWelcome[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerAbort[src], C_PlayerAbort, 
-			  N_PlayerAbort[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB, 
-			  N_PlayerAbortCancel[src], C_PlayerAbortCancel, 
-			  N_PlayerAbortCancel[dst]);
-      /* now color resources */
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerHelmetColor[src], C_PlayerColor,
-			  N_PlayerHelmetColor[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerFaceColor[src], C_PlayerColor,
-			  N_PlayerFaceColor[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerBodyColor[src], C_PlayerColor,
-			  N_PlayerBodyColor[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerHandsFeetColor[src], C_PlayerColor,
-			  N_PlayerHandsFeetColor[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerArmsLegsColor[src], C_PlayerColor,
-			  N_PlayerArmsLegsColor[dst]);
-      copy_database_entry(disp_DB[disp], &player_DB,
-			  N_PlayerBackpackColor[src], C_PlayerColor,
-			  N_PlayerBackpackColor[dst]);
+      /* copy all player attributes */
+      for (i=0; N_PlayerAttributes[i] != NULL; i++) {
+	copy_database_entry(disp_DB[disp], &player_DB, 
+			    N_PlayerAttributes[i][src], 
+			    C_PlayerAttributes[i], 
+			    N_PlayerAttributes[i][dst]);
+      }
     }
   }
 
@@ -1301,6 +1703,33 @@ create_player_database (config)
 
 
 
+/*
+ * public function: set_setup_communication
+ */
+#ifdef __STDC__
+void
+set_setup_communication (int type)
+#else
+void
+set_setup_communication (type)
+     int type;
+#endif
+{
+  switch (type) {
+  case CM_None:
+    create_player_database = local_create_player_DB;
+    break;
+  case CM_Parent:
+    create_player_database = parent_create_player_DB;
+    break;
+  case CM_Child:
+    create_player_database = child_create_player_DB;
+    break;
+  }
+}
+
+
+
 /*
  * public function merge_default_databases
  */
@@ -1313,21 +1742,22 @@ merge_default_databases (config)
      XBConfig *config;
 #endif
 {
-  /* merge default display db into default db overriding */
-  XrmMergeDatabases(disp_DB[config->default_disp], &default_DB);
-
-  /* set display database to new merged one */
-  disp_DB[config->default_disp] = default_DB;
+  if (config->default_disp >= 0) {
+    /* merge default display db into default db overriding */
+    XrmMergeDatabases(disp_DB[config->default_disp], &default_DB);
+    
+    /* set display database to new merged one */
+    disp_DB[config->default_disp] = default_DB;
+  }
 
   /* ovveride new default_DB with player DB */
-  XrmMergeDatabases(create_player_database(config), &default_DB);
+  XrmMergeDatabases((*create_player_database)(config), &default_DB);
 
   /* override new default database with commandline database */
   XrmMergeDatabases(arg_DB, &default_DB);
 }
 
 
-
 /*
  * public funtion setup_from_database
  */
@@ -1347,7 +1777,7 @@ setup_from_database (setup)
   setup->max_lives = db_get_int(default_DB, 1, MAX_LIVES,
 				N_NumberOfLives, C_NumberOfLives);
   setup->random_mode = db_get_bool(default_DB, N_RandomLevelOrder, C_Random);
-  setup->start_level = db_get_int(default_DB, 0, LEVEL_MAX-1, 
+  setup->start_level = db_get_int(default_DB, 0, levelMax-1, 
 				   N_StartingLevel, C_StartingLevel );
   setup->sound_flag = db_get_bool(default_DB, N_BellSound, C_BellSound);
   setup->frame_time = 1000000/db_get_int(default_DB, 2, 100, 
@@ -1359,12 +1789,103 @@ setup_from_database (setup)
   setup->color_mode = db_get_bool(default_DB, N_AllowColorMode, 
 				  C_AllowColorMode);
   /* use level setzen */
-  for (i=0; i<LEVEL_MAX; i++) {
+  for (i=0; i<levelMax; i++) {
     setup->use_level[i] = db_get_bool(default_DB, get_level_res_name(i), C_UseLevel);
   }
 }
 
 
+/*
+ * public function keys_from_database
+ */ 
+#ifdef __STDC__
+int
+keys_from_database (int disp,
+		    int player,
+		    KeyPressDefine *keydef,
+		    PlayerAction *pa)
+#else
+int
+keys_from_database (disp, player, keydef, pa)
+     int disp, player;
+     KeyPressDefine *keydef;
+     PlayerAction *pa;
+#endif
+{
+  int offset = 0;
+  player -= MAX_PLAYER;
+
+#ifdef DEBUG
+  fprintf(stderr, "keys_from_database %d %d\n",disp,player);
+#endif
+
+  /* pause key */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerPauseKey[player],
+					C_PlayerPauseKey);
+  keydef[offset].addr   = &(pa->pause);
+  keydef[offset].value  = TRUE;
+  offset++;
+  /* go up */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerUpKey[player],
+					C_PlayerUpKey);;
+  keydef[offset].addr   = &(pa->dir);
+  keydef[offset].value  = GoUp;
+  offset++;
+  /* go right */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerRightKey[player],
+					C_PlayerRightKey);;
+  keydef[offset].addr   = &(pa->dir);
+  keydef[offset].value  = GoRight;
+  offset++;
+  /* go down */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerDownKey[player],
+					C_PlayerDownKey);;
+  keydef[offset].addr   = &(pa->dir);
+  keydef[offset].value  = GoDown;
+  offset++;
+  /* go left */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerLeftKey[player],
+					C_PlayerLeftKey);;
+  keydef[offset].addr   = &(pa->dir);
+  keydef[offset].value  = GoLeft;
+  offset++;
+  /* go stop */
+  keydef[offset].keysym = db_get_string(disp_DB[disp],N_PlayerStopKey[player],
+					C_PlayerStopKey);;
+  keydef[offset].addr   = &(pa->dir);
+  keydef[offset].value  = GoStop;
+  offset++;
+  /* drop bomb */
+  keydef[offset].keysym = db_get_string(disp_DB[disp],N_PlayerBombKey[player],
+					C_PlayerBombKey);;
+  keydef[offset].addr   = &(pa->bomb);
+  keydef[offset].value  = TRUE;
+  offset++;
+  /* special key */
+  keydef[offset].keysym = db_get_string(disp_DB[disp],N_PlayerSpecialKey[player],
+					C_PlayerSpecialKey);;
+  keydef[offset].addr   = &(pa->special);
+  keydef[offset].value  = TRUE;
+  offset++;
+  /* abort key */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], N_PlayerAbortKey[player],
+					C_PlayerAbortKey);;
+  keydef[offset].addr   = &(pa->abort);
+  keydef[offset].value  = ABORT_TRUE;
+  offset++;
+  /* abort cancel key */
+  keydef[offset].keysym = db_get_string(disp_DB[disp], 
+					N_PlayerAbortCancelKey[player],
+					C_PlayerAbortCancelKey);;
+  keydef[offset].addr   = &(pa->abort);
+  keydef[offset].value  = ABORT_CANCEL;
+  offset++;
+  
+  return offset;
+}
+
+
+
 /*
  * public function color_mode_from_database
  */
@@ -1405,11 +1926,13 @@ override_from_database (disp)
  */
 #ifdef __STDC__
 void
-player_strings_from_database (PlayerStrings *st)
+player_strings_from_database (PlayerStrings *st, 
+			      XBConfig *config)
 #else
 void
-player_strings_from_database (st)
+player_strings_from_database (st, config)
      PlayerStrings *st;
+     XBConfig *config;
 #endif
 {
   int player;
@@ -1418,19 +1941,51 @@ player_strings_from_database (st)
 
   /* first get names for all players */
   for (player=0; player<MAX_PLAYER; player++) {
-    st[player].name = db_get_string(default_DB, N_Player[player], C_Player);
+    st[player].tag = db_get_string(default_DB, N_Player[player], C_Player);
+  }
+  /* if in team mode combine names */
+  switch (config->team_mode) {
+  case TM_Single:
+    for (player=0; player<MAX_PLAYER; player ++) {
+      st[player].name = st[player].tag;
+    }
+    break;
+  case TM_Team:
+    for (player=0; player<MAX_PLAYER; player += 2) {
+      sprintf(line, "%s & %s", st[player].tag, st[player+1].tag);
+      st[player].name = st[player+1].name = dup_string(line, strlen(line)); 
+    }
+    break;
+  case TM_Double:
+    for (player=0; player<MAX_PLAYER; player ++) {
+      st[player + config->num_player/2].name 
+	= st[player + config->num_player/2].tag 
+	= st[player].name = st[player].tag;
+    }
+    break;
+  } 
+
+  /* set pause string */
+  for (player=0; player<MAX_PLAYER; player++) {
+    sprintf(line, "Game paused by %s", st[player].tag);
+    st[player].pause = dup_string(line, strlen(line));
   }
 
-  /* now create database with default strings */
+  /* now create database with default strings (part I)*/
   for (player=0; player<MAX_PLAYER; player++) {
-    sprintf(line, "%s wins", st[player].name);
+    if (config->team_mode == TM_Single) {
+      sprintf(line, "%s wins", st[player].name);
+    } else {
+      sprintf(line, "%s win", st[player].name);
+    }
     XrmPutStringResource(&string_DB, D_PlayerWinLevel[player],line);
-    sprintf(line, "Abort requested by %s", st[player].name);
+    sprintf(line, "Abort requested by %s", st[player].tag);
     XrmPutStringResource(&string_DB, D_PlayerAbort[player], line);
-    sprintf(line, "%s cancels abort", st[player].name);
+    sprintf(line, "%s cancels abort", st[player].tag);
     XrmPutStringResource(&string_DB, D_PlayerAbortCancel[player], line);
   }
 
+
   /* merge with default database */
   XrmCombineDatabase (string_DB, &default_DB, FALSE);
 
@@ -1478,10 +2033,8 @@ create_display_database (disp)
     XrmPutStringResource(&disp_def_DB, ptr->name, ptr->value);
   }
 
-#if 1
   /* get default from applications default file */
   XrmMergeDatabases (XrmGetFileDatabase(file_appl_def), &disp_def_DB);
-#endif  
 
   /* get database from server created by xrdb */
   if (XResourceManagerString(dpy[disp]) != NULL) {
@@ -1557,6 +2110,7 @@ get_color_resources (disp, dc)
 			    C_ExplosionColor2);
   dc->expl3      = db_get_string(disp_DB[disp], N_ExplosionColor3, 
 				 C_ExplosionColor3);
+  dc->bomb       = db_get_string(disp_DB[disp], N_BombColor, C_BombColor);
 }
 
 
@@ -1592,6 +2146,27 @@ get_player_color_resources (pc)
 }
 
 
+/*
+ * public function destroy_databases
+ */
+#ifdef __STDC__
+void
+delete_databases (int num_disp)
+#else
+void
+delete_databases (num_disp)
+     int num_disp;
+#endif
+{
+  int disp;
+  
+  for (disp = 0; disp < num_disp; disp ++) {
+    XrmDestroyDatabase(disp_DB[disp]);
+    disp_DB[disp] = NULL;
+  }
+  default_DB = NULL;
+}
+
 
 /*
  * end of file xsetup.c
